package net.nanex;
import java.io.IOException;
import java.nio.file.Files;
import java.io.File;

public class NxCoreClass {
	static {
	    try {
	        System.loadLibrary("NxCoreJava");
	    } catch (UnsatisfiedLinkError e) {
			String libName;
			if(System.getProperty("os.name").contains("Linux")) {
				if(System.getProperty("os.arch").contains("arm")||System.getProperty("os.arch").contains("aarch64"))
					libName="libNxCoreJava_aarch64.so";
				else
					libName="libNxCoreJava.so";
			}else
				libName="NxCoreJava.dll";
			File tempDir;
			try {
				tempDir = Files.createTempDirectory("_libTemp").toFile();
				tempDir.deleteOnExit();
				File tempLib = new File(tempDir, libName);
				java.io.InputStream in = NxCoreClass.class.getResourceAsStream(libName);
				Files.copy(in, tempLib.toPath());
				tempLib.deleteOnExit();
				System.load(tempLib.getAbsolutePath());
			} catch (IOException e1) {
				e1.printStackTrace();
				System.exit(0);
			}
	    }
	}
	
	public native String GetDefinedString(int ixTable,int ixString);
	public native String GetTapeName();
	public native int GetSymbolAtom32();
	public native int ExcludeOpraExch(int opraId);
	public native int ListTapes(int controlFlags);
	public native int ListAPIDLLs(int controlFlags);
	public native int SpinSymbols(int ListedExg, int controlFlags, int spinID,int tempUserData);
	public native int ProcessTape(String StringNxCoreLib,long pMemoryAddr,int controlFlags, int UserData);
	public native int LoadNxCore(String StringNxCoreLib);
	public native int APIVersion();
	public native void UnloadNxCore();
	public native int SaveState(String szStateFilename, int controlFlags);
	public native int PriceConvert(int lPrice,int PriceType,int PriceTypeNew);
	public native String PriceFormat(int lPrice,int PriceType,int expandWidth, boolean bCommas);
	public native void ProcessReturnValue(int returnValue);
	public native boolean FilterSymbols(String symbolArray[]);
	public native boolean FilterSymbols(String symbol);
	public native boolean FilterSymbolsRange(String startSymbol,String endSymbol);	
	public native boolean ExcludeSymbols(String symbolArray[]);
	public native boolean ExcludeSymbols(String symbol);	
	public NxCoreStateExgQuotes StateGetExgQuotes(int exg,String symbol,NxDate strikeDate,int strikePrice,int PutCall) {
		return StateGetExgQuotes( exg, String.format("%s%d%c%d",symbol,strikeDate.NDays, PutCall==0? 'C' : 'P',strikePrice));
	}
	public NxCoreStateMMQuotes StateGetMMQuotes(int ReportingExg,int exg,String symbol,NxDate strikeDate,int strikePrice,int PutCall) {
		return StateGetMMQuotes( ReportingExg, exg, String.format("%s%d%c%d",symbol,strikeDate.NDays, PutCall==0? 'C' : 'P',strikePrice));
	}
	public NxCoreStateTrade StateGetLastTrade(int exg,String symbol,NxDate strikeDate,int strikePrice,int PutCall) {
		return StateGetLastTrade(exg, String.format("%s%d%c%d",symbol,strikeDate.NDays, PutCall==0? 'C' : 'P',strikePrice));
	}
	public NxCoreStateExgQuotes StateGetExgQuotes(int exg,String symbol){
		NxCoreStateExgQuotes StateExgQuotes = new NxCoreStateExgQuotes();
		if(NativeGetExgQuotes(exg, symbol)!=0)
			return null;
		StateExgQuotes.StateQuoteCount=m_State.getShort(0);
		StateExgQuotes.PriceType=m_State.get(2);
		StateExgQuotes.NasdaqBidTick=m_State.get(3);
		StateExgQuotes.BestAskExg=m_State.getShort(4);
		StateExgQuotes.BestBidExg=m_State.getShort(6);
		for(int i=0;i<StateExgQuotes.StateQuoteCount;i++){
			StateExgQuotes.StateExgQuotes[i]=new NxCoreStateExgQuote();
			StateExgQuotes.StateExgQuotes[i].AskPrice=m_State.getInt(8+i*20);
			StateExgQuotes.StateExgQuotes[i].BidPrice=m_State.getInt(12+i*20);
			StateExgQuotes.StateExgQuotes[i].AskSize=m_State.getInt(16+i*20);
			StateExgQuotes.StateExgQuotes[i].BidSize=m_State.getInt(20+i*20);
			StateExgQuotes.StateExgQuotes[i].ReportingExg=m_State.getShort(24+i*20);
			StateExgQuotes.StateExgQuotes[i].QuoteCondition=m_State.get(26+i*20);
		}
		return StateExgQuotes;
	}
	public NxCoreStateMMQuotes StateGetMMQuotes(int ReportingExg,int exg,String symbol){
		NxCoreStateMMQuotes StateMMQuotes= new NxCoreStateMMQuotes();
		String MMStrings[]=NativeGetMMQuotes(ReportingExg, exg, symbol);
		if(MMStrings.length==0)
			return null;
		StateMMQuotes.StateQuoteCount=m_State.getShort(0);
		StateMMQuotes.PriceType=m_State.get(2);
		StateMMQuotes.NasdaqBidTick=m_State.get(3);
		for(int i=0;i<StateMMQuotes.StateQuoteCount;i++){
			StateMMQuotes.StateMMQuotes[i]=new NxCoreStateMMQuote();
			StateMMQuotes.StateMMQuotes[i].AskPrice=m_State.getInt(8+i*32);
			StateMMQuotes.StateMMQuotes[i].BidPrice=m_State.getInt(12+i*32);
			StateMMQuotes.StateMMQuotes[i].AskSize=m_State.getInt(16+i*32);
			StateMMQuotes.StateMMQuotes[i].BidSize=m_State.getInt(20+i*32);
			StateMMQuotes.StateMMQuotes[i].pnxStringMarketMaker.String=MMStrings[i];
			StateMMQuotes.StateMMQuotes[i].MarketMakerType=m_State.get(32+i*32);
			StateMMQuotes.StateMMQuotes[i].QuoteType=m_State.get(33+i*32);
		}
		return StateMMQuotes;
	}
	public String PriceFormat(int lPrice,int PriceType){
		return PriceFormat(lPrice, PriceType, 0, false);
	}
	public String PriceFormat(int lPrice,int PriceType,int expandWidth){
		return PriceFormat(lPrice, PriceType, expandWidth, false);
	}
	public double PriceToDouble(int lPrice, int PriceType){
		if(PriceType<20)
			return (double)lPrice*Math.pow(10.0,(double)PriceType-10.0);
		return (double)lPrice*Math.pow(2.0,(double)PriceType-30.0);
	}
	public NxCoreStateTrade StateGetLastTrade(int exg,String symbol){
		NxCoreStateTrade StateTrade = new NxCoreStateTrade();
		if(NativeGetLastTrade(exg, symbol)!=0)
			return null;
		StateTrade.TotalVolume=m_State.getLong(0);
		StateTrade.TickVolume=m_State.getInt(8);
		StateTrade.PriceType=m_State.get(12);
		StateTrade.PriceFlags=m_State.get(13);
		StateTrade.ConditionFlags=m_State.get(14);
		StateTrade.VolumeType=m_State.get(15);
		StateTrade.Open=m_State.getInt(16);
		StateTrade.High=m_State.getInt(20);
		StateTrade.Low=m_State.getInt(24);
		StateTrade.Last=m_State.getInt(28);
		StateTrade.NetChange=m_State.getInt(32);
		StateTrade.Price=m_State.getInt(36);
		StateTrade.Threshold=m_State.getInt(40);
		StateTrade.Tick=m_State.getInt(44);
		return StateTrade;
	}
	public void DateFromNDays(NxDate pnxDate){
		NativeDateFromNDays(pnxDate.NDays);
		pnxDate.Year=m_Date.getShort(4)& 0xffff;
		pnxDate.Month=m_Date.get(6)& 0xff;
		pnxDate.Day=m_Date.get(7)& 0xff;
		pnxDate.DSTIndicator=m_Date.get(8)& 0xff;
		pnxDate.DayOfWeek=m_Date.get(9)& 0xff;
		pnxDate.DayOfYear=m_Date.getShort(10);
	}
	public void DateFromYMD(NxDate pnxDate){
		NativeDateFromYMD(pnxDate.Year,pnxDate.Month,pnxDate.Day);
		pnxDate.NDays=m_Date.getInt(0)& 0xffffffff;
		pnxDate.DSTIndicator=m_Date.get(8)& 0xff;
		pnxDate.DayOfWeek=m_Date.get(9)& 0xff;
		pnxDate.DayOfYear=m_Date.getShort(10)& 0xffff;
	}
	public int GetMajorVersion(int soVersion) {
		return (soVersion) >> 24;
	}
	public int GetMinorVersion(int soVersion) {
		return (soVersion >> 16) & 0x000000ff;
	}
	public int GetBuildVersion(int soVersion) {
		return soVersion & 0x0000ffff;
	}

	//Default callback functions
	public int OnUserListTapes(NxCoreTapeFile NxTF){
		return 0;
	}
	public int OnUserListAPI(NxCoreAPIDLLFile NxDF){
		return 0;
	}
	public int OnUserSymbolSpin( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		return 0;
	}
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		return 0;
	}
	
	public class defines{
		public static final int NxDATE_DST_UNKNOWN = 0;
		public static final int NxDATE_DST_ON = 1;
		public static final int NxDATE_DST_OFF = 2;
		public static final int NxDATE_DST_ONTODAY = 3;
		public static final int NxDATE_DST_OFFTODAY = 4;
		public static final int NxDOW_SUNDAY = 0;
		public static final int NxDOW_MONDAY = 1;
		public static final int NxDOW_TUESDAY = 2;
		public static final int NxDOW_WEDNESDAY = 3;
		public static final int NxDOW_THURSDAY = 4;
		public static final int NxDOW_FRIDAY = 5;
		public static final int NxDOW_SATURDAY = 6;
		
		public static final int NxCLOCK_NOCHANGE = 0;
		public static final int NxCLOCK_CLOCK = 1;
		public static final int NxCLOCK_SECOND = 2;
		public static final int NxCLOCK_MINUTE = 3;
		public static final int NxCLOCK_HOUR = 4;
		
		public static final int NxCORESTATUS_RUNNING = 0;
		public static final int NxCORESTATUS_INITIALIZING = 1;
		public static final int NxCORESTATUS_COMPLETE = 2;
		public static final int NxCORESTATUS_SYNCHRONIZING = 3;
		public static final int NxCORESTATUS_ERROR = 4;
		public static final int NxCORESTATUS_WAITFORCOREACCESS = 5;
		public static final int NxCORESTATUS_RESTARTING_TAPE = 6;
		public static final int NxCORESTATUS_LOADED_STATE = 7;
		public static final int NxCORESTATUS_SAVING_STATE = 8;
		public static final int NxCORESTATUS_SYMBOLSPIN = 9;
		public static final int NxCORESTATUS_TAPEOPEN = 10;
		
		public static final int NxCSRUNMODE_SRC_DISK_FILE = 0;
		public static final int NxCSRUNMODE_SRC_ACCESS_FILE = 1;
		public static final int NxCSRUNMODE_SRC_ACCESS_MEMORY = 2;
		
		public static final int NxCSRUNMODE_SRC_ACCESS_MB_OLDEST = 2;
		public static final int NxCSRUNMODE_SRC_ACCESS_MB_LOWER = 3;
		public static final int NxCSRUNMODE_SRC_ACCESS_MB_UPPER = 4;
		public static final int NxCSRUNMODE_SRC_ACCESS_MB_SECOND = 5;
		public static final int NxCSRUNMODE_SRC_ACCESS_MB_CURRENT = 6;
		
		public static final int NxCSSAVESTATE_CAPTURE = 0;
		public static final int NxCSSAVESTATE_COMPLETE = 1;
		public static final int NxCSSAVESTATE_ERR_CREATEFILE = -1;
		public static final int NxCSSAVESTATE_ERR_DISKSPACE = -2;
		public static final int NxCSSAVESTATE_ERR_MEMORY = -3;
		
		public static final int NxCSSYMBOLSPIN_STARTING = 0;
		public static final int NxCSSYMBOLSPIN_COMPLETE = 1;
		public static final int NxOPRAID_SELECT_CHAN = 0x00400000;
		public static final int NxOPRAID_ONLY = 0x00800000;
		public static final int NxOPRAID_BBO_01 = 0x00000001;
		public static final int NxOPRAID_BBO_02 = 0x00000002;
		public static final int NxOPRAID_BBO_03 = 0x00000004;
		public static final int NxOPRAID_BBO_04 = 0x00000008;
		public static final int NxOPRAID_FULL_01 = 0x00000010;
		public static final int NxOPRAID_FULL_02 = 0x00000020;
		public static final int NxOPRAID_FULL_03 = 0x00000040;
		public static final int NxOPRAID_FULL_04 = 0x00000080;
		public static final int NxOPRAID_FULL_05 = 0x00000100;
		public static final int NxOPRAID_FULL_06 = 0x00000200;
		public static final int NxOPRAID_FULL_07 = 0x00000400;
		public static final int NxOPRAID_FULL_08 = 0x00000800;
		public static final int NxOPRAID_FULL_09 = 0x00001000;
		public static final int NxOPRAID_FULL_10 = 0x00002000;
		public static final int NxOPRAID_FULL_11 = 0x00004000;
		public static final int NxOPRAID_FULL_12 = 0x00008000;
		public static final int NxOPRAID_FULL_13 = 0x00010000;
		public static final int NxOPRAID_FULL_14 = 0x00020000;
		public static final int NxOPRAID_FULL_15 = 0x00040000;
		public static final int NxOPRAID_FULL_16 = 0x00080000;
		public static final int NxPERMID_REMOVE = 0x00100000;
		
		public static final int NxQTE_NASDBIDTICK_NA = 0;
		public static final int NxQTE_NASDBIDTICK_UP = 1;
		public static final int NxQTE_NASDBIDTICK_DN = 2;
		public static final int NxQTE_NASDBIDTICK_NO = 3;
		public static final int NxQTE_REFRESH_REALTIME = 0;
		public static final int NxQTE_REFRESH_SOURCE = 1;
		public static final int NxQTE_REFRESH_MISMATCH = 2;
		public static final int NxQTE_REFRESH_SYNCHRONIZE = 3;
		
		public static final int NxBBOCHANGE_BIDEXG = 0x01;
		public static final int NxBBOCHANGE_BIDPRICE = 0x02;
		public static final int NxBBOCHANGE_BIDSIZE = 0x04;
		public static final int NxBBOCHANGE_ASKEXG = 0x10;
		public static final int NxBBOCHANGE_ASKPRICE = 0x20;
		public static final int NxBBOCHANGE_ASKSIZE = 0x40;
		
		public static final int NxMMT_UNKNOWN = 0;
		public static final int NxMMT_REGULAR = 1;
		public static final int NxMMT_PRIMARY = 2;
		public static final int NxMMT_PASSIVE = 3;
		
		public static final int NxMMQT_UNKNOWN = 0;
		public static final int NxMMQT_REGULAR = 1;
		public static final int NxMMQT_PASSIVE = 2;
		public static final int NxMMQT_SYNDICATE = 3;
		public static final int NxMMQT_PRESYNDICATE = 4;
		public static final int NxMMQT_PENALTY = 5;
		
		public static final int NxTPF_SETLAST = 0x01;
		public static final int NxTPF_SETHIGH = 0x02;
		public static final int NxTPF_SETLOW = 0x04;
		public static final int NxTPF_SETOPEN = 0x08;
		public static final int NxTPF_EXGINSERT = 0x10;
		public static final int NxTPF_EXGCANCEL = 0x20;
		public static final int NxTPF_SETTLEMENT = 0x40;
		
		public static final int NxTCF_NOLAST = 0x01;
		public static final int NxTCF_NOHIGH = 0x02;
		public static final int NxTCF_NOLOW = 0x04;
		
		public static final int NxTVT_INCRVOL = 0;
		public static final int NxTVT_NONINCRVOL = 1;
		public static final int NxTVT_TOTALVOL = 2;
		public static final int NxTVT_TOTALVOLx100 = 3;
		
		public static final int NxRTA_SIGHL_EQ = 0;
		public static final int NxRTA_SIGHL_LOWEST = 1;
		public static final int NxRTA_SIGHL_LOW = 2;
		public static final int NxRTA_SIGHL_HIGH = 3;
		public static final int NxRTA_SIGHL_HIGHEST = 4;
		
		public static final int NxRTA_QMTYPE_NONE = 0;
		public static final int NxRTA_QMTYPE_BID = 1;
		public static final int NxRTA_QMTYPE_ASK = 2;
		public static final int NxRTA_QMTYPE_INSIDE = 3;
		public static final int NxRTA_QMTYPE_BELOWBID = 4;
		public static final int NxRTA_QMTYPE_ABOVEASK = 5;
		
		public static final int NxRTA_QTEMATCHFLAG_OLDER = 0x01;
		public static final int NxRTA_QTEMATCHFLAG_CROSSED = 0x02;
		
		public static final int NxCFT_UNKNOWN = 0;
		public static final int NxCFT_64BIT = 1;
		public static final int NxCFT_32BIT = 2;
		public static final int NxCFT_STRINGZ = 3;
		public static final int NxCFT_DOUBLE = 4;
		public static final int NxCFT_PRICE = 5;
		public static final int NxCFT_DATE = 6;
		public static final int NxCFT_TIME = 7;
		public static final int NxCFT_NxSTRING = 8;
		public static final int NxCFT_STRING_IDX = 9;
		public static final int NxCFT_STRING_MAP = 10;
		
		public static final int NxSS_ADD = 0;
		public static final int NxSS_DEL = 1;
		public static final int NxSS_MOD = 2;
		
		public static final int NxSPIN_ALL_EXGS = 65535;
		public static final int NxSPIN_NON_OPTIONS = 0;
		public static final int NxSPIN_OPTIONS = 1;
		
		public static final int NxMSG_STATUS = 0;
		public static final int NxMSG_EXGQUOTE = 1;
		public static final int NxMSG_MMQUOTE = 2;
		public static final int NxMSG_TRADE = 3;
		public static final int NxMSG_CATEGORY = 4;
		public static final int NxMSG_SYMBOLCHANGE = 5;
		public static final int NxMSG_SYMBOLSPIN = 6;
		
		public static final int NxSTATETYPE_OHLCTRADE = 1;
		public static final int NxSTATETYPE_EXGQUOTES = 2;
		public static final int NxSTATETYPE_MMQUOTES = 3;
		
		public static final int NxLTF_SEARCH_LOCAL = 0;
		public static final int NxLTF_SEARCH_NETWORK = 0x00000001;
		public static final int NxLTF_SEARCH_REMOVEABLE = 0x00000002;
		
		public static final int NxST_PERMID = 2;
		public static final int NxST_EXCHANGE = 3;
		public static final int NxST_TRADECONDITION = 4;
		public static final int NxST_QUOTECONDITION = 5;
		public static final int NxST_CATEGORY = 6;
		public static final int NxST_HALTSTATUS = 7;
		public static final int NxST_HALTREASONTYPE = 8;
		public static final int NxST_OPENINDICATIONTYPE = 9;
		public static final int NxST_ORDERIMBALANCETYPE = 10;
		public static final int NxST_TRADEREPORTREASON = 11;
		public static final int NxST_EXGCORRECTIONMAP = 12;
		public static final int NxST_SYMBOLCHANGETYPE = 13;
		public static final int NxST_OHLCFLAGS = 14;
		public static final int NxST_EXDIVIDENDATTRIB = 15;
		public static final int NxST_OPRAEXCHANGELIST = 16;
		public static final int NxST_NASDBIDTICK = 17;
		public static final int NxST_MARKETMAKERTYPE = 18;
		public static final int NxST_MMQUOTETYPE = 19;
		public static final int NxST_REFRESHTYPE = 20;
		public static final int NxST_EXPIRATIONCYCLE = 21;
		public static final int NxST_VOLUMETYPE = 22;
		public static final int NxST_QTEMATCHTYPE = 23;
		public static final int NxST_QTEMATCHFLAGS = 24;
		public static final int NxST_SIGHIGHLOWTYPE = 25;
		public static final int NxST_BBOCHANGEFLAGS = 26;
		public static final int NxST_TRADECONDFLAGS = 27;
		public static final int NxST_PRICEFLAGS = 28;
		public static final int NxST_SETTLEFLAGMAP = 29;
		public static final int NxST_SYMBOLFNCLSTATUSMAP = 30;
		public static final int NxST_EQUITYCLASSIFICATIONS = 31;
		public static final int NxST_SYMBOLTYPES = 32;
		
		public static final int NxCF_EXCLUDE_QUOTES = 0x00000001;
		public static final int NxCF_EXCLUDE_QUOTES2 = 0x00000002;
		public static final int NxCF_EXCLUDE_OPRA = 0x00000004;
		public static final int NxCF_FAVOR_NBBO = 0x00000008;
		public static final int NxCF_MEMORY_ADDRESS = 0x04000000;
		public static final int NxCF_EXCLUDE_CRC_CHECK = 0x08000000;
		
		public static final int NxSAVESTATE_GRADUALLY = 0;
		public static final int NxSAVESTATE_ONEPASS = 0x00000001;
		public static final int NxSAVESTATE_CANCEL = 0x00000002;
		
		public static final int NxCALLBACKRETURN_CONTINUE = 0;
		public static final int NxCALLBACKRETURN_STOP = 1;
		public static final int NxCALLBACKRETURN_RESTART = 2;
		
		public static final int NxAPIERR_NO_ERROR = 0;
		public static final int NxAPIERR_USER_STOPPED = 1;
		public static final int NxAPIERR_NOT_CALLBACK_THREAD = -1;
		public static final int NxAPIERR_BAD_PARAMETERS = -2;
		public static final int NxAPIERR_EXCEPTION = -3;
		public static final int NxAPIERR_OPEN_TAPE_FILE_FAILED = -4;
		public static final int NxAPIERR_INITIALIZE_MEMORY_FAILED = -5;
		public static final int NxAPIERR_NOLISTED_EXG = -6;
		public static final int NxAPIERR_NOSYMBOLS_FOR_LSTEXG = -7;
		public static final int NxAPIERR_NOSESSION_FOR_SYMBOL = -8;
		public static final int NxAPIERR_NODATATYPE_FOR_SESSION = -9;
		public static final int NxAPIERR_NODATA_FOR_REPEXG = -10;
		public static final int NxAPIERR_ZEROED_DATA_FOR_SESSION = -11;
		public static final int NxAPIERR_SAVE_STATE_IN_PROGRESS = -12;
		public static final int NxAPIERR_NOT_SUPPORTED = -13;
		public static final int NxAPIERR_INITALLOC_ERROR = -14;
		public static final int NxAPIERR_NSODALLOC_ERROR = -15;
		
		public static final int LULD_BND_EVT_NONE = 0x00;
		public static final int LULD_BND_EVT_OPENING = 0x01;
		public static final int LULD_BND_EVT_INTRADAY = 0x02;
		public static final int LULD_BND_EVT_RESTATED = 0x03;
		public static final int LULD_BND_EVT_SUSPENDED = 0x04;
		public static final int LULD_BND_EVT_REOPENING = 0x05;
		public static final int LULD_BND_EVT_OUTSIDE_HRS = 0x06;
	}
	
	public class NxCoreTapeFile{
		public String				PathnameStrZ="";
		public int					PathnameLen;
		public int					FilenameLen;
		public int					FileAttributes;
		public long				FileSize;
		public NxDate				TapeDate=new NxDate();
		public String				PermSet="";
	}
	public class NxCoreAPIDLLFile { // 304/304 bytes
		public String				PathnameStrZ="";
		public int					PathnameLen;
		public int					FilenameLen;
		public int					FileAttributes;
		public long				FileSize;
		public NxDate				BuildDate= new NxDate();	// 16/16 bytes
		public NxTime				BuildTime= new NxTime();	// 16/16 bytes
		public int					verMajor;
		public int					verMinor;
		public int					verBuild;
	}
	public class NxDate{
		public int NDays;
		public int Year;
		public int Month;
		public int Day;
		public int DSTIndicator;
		public int DayOfWeek;
		public int DayOfYear;
	}
	public class NxTime{
		public int MsOfDay;
		public int Millisecond;
		public int Second;
		public int Minute;
		public int Hour;
		public int TimeZone;
		public int MsResolution;
	}
	public class NxString{
		public int UserData1;
		public int UserData2;
		public int Atom;
		public String String="";
	}
	public class NxOptionHdr{
		public NxString		  pnxsDateAndStrike = new NxString();
		public NxString		  pnxsUnderlying = new NxString();
		public NxString		pnxsSeriesChain = new NxString();
		public NxString		pnxsSpecialSettle = new NxString();
		public int				exgUnderlying;
		public int				contractUnit;
		public int				strikePrice;
		public int				strikePriceEst;
		public int				PutCall;
		public int				expirationCycle;
		public int				oicStrikeAge;
		public int				nxStrikeMatch;
		public NxDate			nxExpirationDate = new NxDate();
	}
	public class NxCoreHeader{
		public int ListedExg;
		public NxString pnxStringSymbol = new NxString();
		public NxOptionHdr pnxOptionHdr = new NxOptionHdr();
		public NxDate nxSessionDate = new NxDate();
		public NxTime nxExgTimestamp = new NxTime();
		public int ReportingExg;
		public int SessionID;
		public int PermissionID;
	}
	public class NxCoreQuote {
		public int		AskSize;
		public int		BidSize;
		public int		AskSizeChange;
		public int		BidSizeChange;
		public int		AskPrice;
		public int		BidPrice;
		public int		AskPriceChange;
		public int		BidPriceChange;
		public int		PriceType;
		public int		Refresh;
		public int		QuoteCondition;
		public int		NasdaqBidTick;
	};
	public class NxCoreExgQuote {
		public NxCoreQuote		coreQuote=new NxCoreQuote();
		public int				BestAskPrice;
		public int				BestBidPrice;
		public int				BestAskPriceChange;
		public int				BestBidPriceChange;
		public int				BestAskSize;
		public int				BestBidSize;
		public int				BestAskSizeChange;
		public int				BestBidSizeChange;
		public int				BestAskExg;
		public int				BestBidExg;
		public int				PrevBestAskExg;
		public int				PrevBestBidExg;
		public int				BestAskCondition;
		public int				BestBidCondition;
		public int				BBOChangeFlags;
		public int				ClosingQuoteFlag;
	};
	public class NxCoreMMQuote {
		public NxCoreQuote			coreQuote=new NxCoreQuote();
		public NxString			pnxStringMarketMaker=new NxString();
		public int		MarketMakerType;
		public int		QuoteType;
	};
	public class NxCTAnalysis {
		public int		FilterThreshold;
		public int		Filtered;
		public int		FilterLevel;
		public int		SigHiLoType;
		public int		alignment;
		public int		SigHiLoSeconds;
		public int		QteMatchDistanceRGN;
		public int		QteMatchDistanceBBO;
		public int		QteMatchTypeRGN;
		public int		QteMatchTypeBBO;
		public int		QteMatchFlagsRGN;
		public int		QteMatchFlagsBBO;
	};
	public class NxCoreTrade {
		public int				Price;
		public int				PriceType;
		public int				PriceFlags;
		public int				TradeCondition;
		public int				ConditionFlags;
		public int				VolumeType;
		public int				BATECode;
		public int				Size;
		public int				ExgSequence;
		public int				RecordsBack;
		public NxCTAnalysis	nxAnalysis=new NxCTAnalysis();
		public long			TotalVolume;
		public int				TickVolume;
		public int				Open;
		public int				High;
		public int				Low;
		public int				Last;
		public int				Tick;
		public int				NetChange;
		public int[]			ExtTradeConditions={0,0,0,0};
	};
	public class NxPrice{
		public int				Price;
		public int				PriceType;
	}
	public class StringTableItem{
		public int				ixTable;
		public int				idString;
	}
	public class NxCategoryFieldData{
		public long			i64Bit;									// 1: NxCFT_64BIT
		public int				i32Bit;									// 2: NxCFT_32BIT
		public String			StringZ;								// 3: NxCFT_STRINGZ
		public double			Double;									// 4: NxCFT_DOUBLE
		public NxPrice			nxPrice= new NxPrice();					// 5: NxCFT_PRICE
		public NxDate			nxDate = new NxDate();					// 6: NxCFT_DATE
		public NxTime			nxTime = new NxTime();					// 7: NxCFT_TIME
		public NxString		pnxString = new NxString();				// 8: NxCFT_NxSTRING
		public StringTableItem	stringTableItem = new StringTableItem();// 9: NxCFT_STRING_IDX, 10:NxCFT_STRING_MAP // 8/8 bytes
	}
	public class NxCategoryField {
		public String			FieldName="";
		public String			FieldInfo="";
		public int				Set;
		public int				FieldType;
		public NxCategoryFieldData	data=new NxCategoryFieldData();
	}
	public class NxCoreCategory {
		public NxString		pnxStringCategory=new NxString();
		public NxCategoryField	pnxFields[]=new NxCategoryField[256];
		public int				NFields;
	};
	public class NxCoreSymbolChange {
		public int				Status;
		public int				StatusChar;
		public NxString		pnxsSymbolOld   = new NxString();
		public NxOptionHdr		pnxOptionHdrOld = new NxOptionHdr();
		public int				ListedExgOld;
	};
	public class NxCoreSymbolSpin {
		public int		SpinID;
		public int		IterLimit;
		public int		SubIterLimit;
		public int		IterCount;
		public int		SubIterCount;
	};
	public class NxCore3Ext {
		public long	 MicrosOfDaySRC;
		public long	 MicrosOfDayPTP1;
		public long	 MicrosOfDayPTP2;
	};
	public class NxCoreStateExgQuote {
		public int				AskPrice;
		public int				BidPrice;
		public int				AskSize;
		public int				BidSize;
		public int				ReportingExg;
		public int				QuoteCondition;
	};
	public class NxCoreStateMMQuote {
		public int				AskPrice;
		public int				BidPrice;
		public int				AskSize;
		public int				BidSize;
		public NxString		pnxStringMarketMaker = new NxString();
		public int				MarketMakerType;
		public int				QuoteType;
	};
	public class NxCoreStateExgQuotes {
		public int					StateQuoteCount;
		public int					PriceType;
		public int					NasdaqBidTick;
		public int					BestAskExg;
		public int					BestBidExg;
		public NxCoreStateExgQuote	StateExgQuotes[] = new NxCoreStateExgQuote[24];
	};
	public class NxCoreStateMMQuotes {
		public int					StateQuoteCount;
		public int					PriceType;
		public int					NasdaqBidTick;
		public NxCoreStateMMQuote	StateMMQuotes[] = new NxCoreStateMMQuote[512];
	};
	public class NxCoreStateTrade {
		public long				TotalVolume;
		public int					TickVolume;
		public int					PriceType;
		public int					PriceFlags;
		public int					ConditionFlags;
		public int					VolumeType;
		public int					Open;
		public int					High;
		public int					Low;
		public int					Last;
		public int					NetChange;
		public int					Price;
		public int					Threshold;
		public int					Tick;
	};
	public class NxCoreSystem{
		public int UserData;
		public int DLLVersion;
		public NxDate nxDate=new NxDate();
		public NxTime nxTime=new NxTime();
		public int ClockUpdateInterval;
		public int Status;
		public int StatusData;
	}
	public class NxCoreData{
		public NxCoreExgQuote		ExgQuote = new NxCoreExgQuote();
		public NxCoreMMQuote		MMQuote = new NxCoreMMQuote();
		public NxCoreTrade			Trade = new NxCoreTrade();
		public NxCoreCategory		Category = new NxCoreCategory();
		public NxCoreSymbolChange	SymbolChange = new NxCoreSymbolChange();
		public NxCoreSymbolSpin	SymbolSpin = new NxCoreSymbolSpin();
	}
	public class NxCoreMessage{
		public NxCoreHeader coreHeader = new NxCoreHeader();
		public NxCoreData coreData = new NxCoreData();
		public int MessageType;
		public NxCore3Ext pnxCore3Ext = null;
	}
	
	private java.nio.ByteBuffer m_Sys;
	private java.nio.ByteBuffer m_NxOptionHdr;
	private java.nio.ByteBuffer m_coreHeader;
	private java.nio.ByteBuffer m_ExgQuote;
	private java.nio.ByteBuffer m_MMQuote;
	private java.nio.ByteBuffer m_Trade;
	private java.nio.ByteBuffer m_Category;
	private java.nio.ByteBuffer m_SymbolChange;
	private java.nio.ByteBuffer m_SymbolChangeOptionHdr;
	private java.nio.ByteBuffer m_SymbolSpin;
	private java.nio.ByteBuffer m_SymbolUserData;
	private java.nio.ByteBuffer m_ContractUserData;
	private java.nio.ByteBuffer m_UnderlyingUserData;
	private java.nio.ByteBuffer m_State;
	private java.nio.ByteBuffer m_Date;
	private java.nio.ByteBuffer m_TapeFileInfo;
	private java.nio.ByteBuffer m_APIDLLFileInfo;
	private java.nio.ByteBuffer m_NxCore3Ext;
	public NxCoreClass(){
		m_Sys=java.nio.ByteBuffer.allocateDirect( 104 );
		m_NxOptionHdr=java.nio.ByteBuffer.allocateDirect( 64 );//48 if 32-bit
		m_coreHeader=java.nio.ByteBuffer.allocateDirect( 56 );//48 if 32-bit
		m_ExgQuote=java.nio.ByteBuffer.allocateDirect( 88 );
		m_MMQuote=java.nio.ByteBuffer.allocateDirect(52 );//48 is 32-bit
		m_Trade=java.nio.ByteBuffer.allocateDirect( 88 );
		m_Category=java.nio.ByteBuffer.allocateDirect( 40*256 );
		m_SymbolChange=java.nio.ByteBuffer.allocateDirect( 32 );
		m_SymbolSpin=java.nio.ByteBuffer.allocateDirect( 20 );
		m_SymbolChangeOptionHdr=java.nio.ByteBuffer.allocateDirect( 64 );//48 if 32-bit
		m_SymbolUserData=java.nio.ByteBuffer.allocateDirect( 10 );
		m_ContractUserData=java.nio.ByteBuffer.allocateDirect( 10 );
		m_UnderlyingUserData=java.nio.ByteBuffer.allocateDirect( 10 );
		m_State=java.nio.ByteBuffer.allocateDirect( 16392 );// 12292 if 32-bit
		m_Date=java.nio.ByteBuffer.allocateDirect( 16);
		m_TapeFileInfo=java.nio.ByteBuffer.allocateDirect( 304 );
		m_APIDLLFileInfo=java.nio.ByteBuffer.allocateDirect( 316 );
		m_NxCore3Ext=java.nio.ByteBuffer.allocateDirect( 24 );
		
		m_Sys.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_NxOptionHdr.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_coreHeader.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_ExgQuote.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_MMQuote.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_Trade.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_Category.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_SymbolChange.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_SymbolSpin.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_SymbolChangeOptionHdr.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_SymbolUserData.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_ContractUserData.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_UnderlyingUserData.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_State.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_Date.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_TapeFileInfo.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_APIDLLFileInfo.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		m_NxCore3Ext.order(java.nio.ByteOrder.LITTLE_ENDIAN);
		for(int i=0;i<256;i++)
			m_NxCoreMsg.coreData.Category.pnxFields[i]=new NxCategoryField();
	}
	
	private NxCoreSystem m_NxCoreSys = new NxCoreSystem();
	private NxCoreMessage m_NxCoreMsg = new NxCoreMessage();
	private NxCoreTapeFile m_NxTF = new NxCoreTapeFile();
	private NxCoreAPIDLLFile m_NxAF = new NxCoreAPIDLLFile();
	private NxCoreSystem m_NxCoreSysSpin = new NxCoreSystem();
	private NxCoreMessage m_NxCoreMsgSpin = new NxCoreMessage();
	
	private void populateCoreHeader(String Symbol){
		m_NxCoreMsg.coreHeader.pnxStringSymbol.String=Symbol;
		m_NxCoreMsg.coreHeader.pnxStringSymbol.UserData1=m_SymbolUserData.getInt(0);
		m_NxCoreMsg.coreHeader.pnxStringSymbol.UserData2=m_SymbolUserData.getInt(4);
		m_NxCoreMsg.coreHeader.pnxStringSymbol.Atom=m_SymbolUserData.getShort(8);
		//8 byte pointer
		m_NxCoreMsg.coreHeader.nxSessionDate.NDays=m_coreHeader.getInt(16);
		m_NxCoreMsg.coreHeader.nxSessionDate.Year=m_coreHeader.getShort(20);
		m_NxCoreMsg.coreHeader.nxSessionDate.Month=m_coreHeader.get(22);
		m_NxCoreMsg.coreHeader.nxSessionDate.Day=m_coreHeader.get(23);
		m_NxCoreMsg.coreHeader.nxSessionDate.DSTIndicator=m_coreHeader.get(24);
		m_NxCoreMsg.coreHeader.nxSessionDate.DayOfWeek=m_coreHeader.get(25);
		m_NxCoreMsg.coreHeader.nxSessionDate.DayOfYear=m_coreHeader.getShort(26);
		//4 byte pad
		m_NxCoreMsg.coreHeader.nxExgTimestamp.MsOfDay=m_coreHeader.getInt(32);
		m_NxCoreMsg.coreHeader.nxExgTimestamp.Millisecond=m_coreHeader.getShort(36);
		m_NxCoreMsg.coreHeader.nxExgTimestamp.Second=m_coreHeader.get(38);
		m_NxCoreMsg.coreHeader.nxExgTimestamp.Minute=m_coreHeader.get(39);
		m_NxCoreMsg.coreHeader.nxExgTimestamp.Hour=m_coreHeader.get(40);
		m_NxCoreMsg.coreHeader.nxExgTimestamp.TimeZone=m_coreHeader.get(41);
		m_NxCoreMsg.coreHeader.nxExgTimestamp.MsResolution=m_coreHeader.getShort(42);
		//4 byte pad
		m_NxCoreMsg.coreHeader.ListedExg=m_coreHeader.getShort(48);
		m_NxCoreMsg.coreHeader.ReportingExg=m_coreHeader.getShort(50);
		m_NxCoreMsg.coreHeader.SessionID=m_coreHeader.get(52);
		//1 byte pad
		m_NxCoreMsg.coreHeader.PermissionID=m_coreHeader.getShort(54);
		if(m_NxCoreMsg.pnxCore3Ext!=null) {
			m_NxCoreMsg.pnxCore3Ext.MicrosOfDaySRC=m_NxCore3Ext.getLong(0);
			m_NxCoreMsg.pnxCore3Ext.MicrosOfDayPTP1=m_NxCore3Ext.getLong(8);
			m_NxCoreMsg.pnxCore3Ext.MicrosOfDayPTP2=m_NxCore3Ext.getLong(16);
		}
	}
	private void populateOptionHeader(String pnxsDateAndStrike,String pnxsUnderlying,String pnxsSeriesChain,String pnxsSpecialSettle){
		if(m_NxCoreMsg.coreHeader.pnxOptionHdr==null)
			m_NxCoreMsg.coreHeader.pnxOptionHdr=new NxOptionHdr();
		if(pnxsDateAndStrike!=null) {
			if(m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike==null)
				m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike=new NxString();
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData1=m_ContractUserData.getInt(0);
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData2=m_ContractUserData.getInt(4);
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.Atom=m_ContractUserData.getShort(8);
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String=pnxsDateAndStrike;
		}
		else
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike=null;
		if(pnxsUnderlying!=null) {
			if(m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying==null)
				m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying=new NxString();
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.String=pnxsUnderlying;
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData1=m_UnderlyingUserData.getInt(0);
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData2=m_UnderlyingUserData.getInt(4);
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.Atom=m_UnderlyingUserData.getShort(8);
			
		}
		else
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying=null;
		if(pnxsSeriesChain!=null) {
			if(m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSeriesChain==null)
				m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSeriesChain=new NxString();
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSeriesChain.String=pnxsSeriesChain;
		}
		else
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSeriesChain=null;
		if(pnxsSpecialSettle!=null) {
			if(m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSpecialSettle==null)
				m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSpecialSettle=new NxString();
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSpecialSettle.String=pnxsSpecialSettle;
		}
		else
			m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSpecialSettle=null;
		m_NxCoreMsg.coreHeader.pnxOptionHdr.exgUnderlying = m_NxOptionHdr.getShort(32);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.contractUnit = m_NxOptionHdr.getShort(34);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice = m_NxOptionHdr.getInt(36);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.strikePriceEst = m_NxOptionHdr.getInt(40);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.PutCall = m_NxOptionHdr.get(44);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.expirationCycle = m_NxOptionHdr.get(45);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.oicStrikeAge = m_NxOptionHdr.get(46);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxStrikeMatch = m_NxOptionHdr.get(47);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.NDays = m_NxOptionHdr.getInt(48);;
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Year = m_NxOptionHdr.getShort(52);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Month = m_NxOptionHdr.get(54);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Day =  m_NxOptionHdr.get(55);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.DSTIndicator = m_NxOptionHdr.get(56);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.DayOfWeek = m_NxOptionHdr.get(57);
		m_NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.DayOfYear = m_NxOptionHdr.getShort(58);
	}
	private void populateCoreHeaderSpin(String Symbol){
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.String=Symbol;
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.UserData1=m_SymbolUserData.getInt(0);
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.UserData2=m_SymbolUserData.getInt(4);
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.Atom=m_SymbolUserData.getShort(8);
		//8 byte pointer
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.NDays=m_coreHeader.getInt(16);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.Year=m_coreHeader.getShort(20);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.Month=m_coreHeader.get(22);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.Day=m_coreHeader.get(23);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.DSTIndicator=m_coreHeader.get(24);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.DayOfWeek=m_coreHeader.get(25);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.DayOfYear=m_coreHeader.getShort(26);
		//4 byte pad
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.MsOfDay=m_coreHeader.getInt(32);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Millisecond=m_coreHeader.getShort(36);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Second=m_coreHeader.get(38);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Minute=m_coreHeader.get(39);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Hour=m_coreHeader.get(40);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.TimeZone=m_coreHeader.get(41);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.MsResolution=m_coreHeader.getShort(42);
		//4 byte pad
		m_NxCoreMsgSpin.coreHeader.ListedExg=m_coreHeader.getShort(48);
		m_NxCoreMsgSpin.coreHeader.ReportingExg=m_coreHeader.getShort(50);
		m_NxCoreMsgSpin.coreHeader.SessionID=m_coreHeader.get(52);
		//1 byte pad
		m_NxCoreMsgSpin.coreHeader.PermissionID=m_coreHeader.getShort(54);
	}
	private void populateOptionHeaderSpin(String pnxsDateAndStrike,String pnxsUnderlying,String pnxsSeriesChain,String pnxsSpecialSettle){
		if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr==null)
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr=new NxOptionHdr();
		if(pnxsDateAndStrike!=null) {
			if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike==null)
				m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike=new NxString();
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData1=m_ContractUserData.getInt(0);
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData2=m_ContractUserData.getInt(4);
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike.Atom=m_ContractUserData.getShort(8);
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String=pnxsDateAndStrike;
		}
		else
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike=null;
		if(pnxsUnderlying!=null) {
			if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying==null)
				m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying=new NxString();
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData1=m_UnderlyingUserData.getInt(0);
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData2=m_UnderlyingUserData.getInt(4);
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying.Atom=m_UnderlyingUserData.getShort(8);
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying.String=pnxsUnderlying;
		}
		else
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying=null;
		if(pnxsSeriesChain!=null) {
			if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSeriesChain==null)
				m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSeriesChain=new NxString();
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSeriesChain.String=pnxsSeriesChain;
		}
		else
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSeriesChain=null;
		if(pnxsSpecialSettle!=null) {
			if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSpecialSettle==null)
				m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSpecialSettle=new NxString();
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSpecialSettle.String=pnxsSpecialSettle;
		}
		else
			m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsSpecialSettle=null;
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.exgUnderlying = m_NxOptionHdr.getShort(32);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.contractUnit = m_NxOptionHdr.getShort(34);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.strikePrice = m_NxOptionHdr.getInt(36);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.strikePriceEst = m_NxOptionHdr.getInt(40);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.PutCall = m_NxOptionHdr.get(44);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.expirationCycle = m_NxOptionHdr.get(45);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.oicStrikeAge = m_NxOptionHdr.get(46);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxStrikeMatch = m_NxOptionHdr.get(47);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.NDays = m_NxOptionHdr.getInt(48);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.Year = m_NxOptionHdr.getShort(52);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.Month = m_NxOptionHdr.get(54);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.Day =  m_NxOptionHdr.get(55);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.DSTIndicator = m_NxOptionHdr.get(56);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.DayOfWeek = m_NxOptionHdr.get(57);
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.nxExpirationDate.DayOfYear = m_NxOptionHdr.getShort(58);
	}
	private void populateExgQuote(){
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.AskSize=m_ExgQuote.getInt(0);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.BidSize=m_ExgQuote.getInt(4);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.AskSizeChange=m_ExgQuote.getInt(8);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.BidSizeChange=m_ExgQuote.getInt(12);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.AskPrice=m_ExgQuote.getInt(16);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.BidPrice=m_ExgQuote.getInt(20);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.AskPriceChange=m_ExgQuote.getInt(24);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.BidPriceChange=m_ExgQuote.getInt(28);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.PriceType=m_ExgQuote.get(32);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.Refresh=m_ExgQuote.get(33);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.QuoteCondition=m_ExgQuote.get(34);
		m_NxCoreMsg.coreData.ExgQuote.coreQuote.NasdaqBidTick=m_ExgQuote.get(35);
		//4 byte pad
		m_NxCoreMsg.coreData.ExgQuote.BestAskPrice=m_ExgQuote.getInt(40);
		m_NxCoreMsg.coreData.ExgQuote.BestBidPrice=m_ExgQuote.getInt(44);
		m_NxCoreMsg.coreData.ExgQuote.BestAskPriceChange=m_ExgQuote.getInt(48);
		m_NxCoreMsg.coreData.ExgQuote.BestBidPriceChange=m_ExgQuote.getInt(52);
		m_NxCoreMsg.coreData.ExgQuote.BestAskSize=m_ExgQuote.getInt(56);
		m_NxCoreMsg.coreData.ExgQuote.BestBidSize=m_ExgQuote.getInt(60);
		m_NxCoreMsg.coreData.ExgQuote.BestAskSizeChange=m_ExgQuote.getInt(64);
		m_NxCoreMsg.coreData.ExgQuote.BestBidSizeChange=m_ExgQuote.getInt(68);
		m_NxCoreMsg.coreData.ExgQuote.BestAskExg=m_ExgQuote.getShort(72);
		m_NxCoreMsg.coreData.ExgQuote.BestBidExg=m_ExgQuote.getShort(74);
		m_NxCoreMsg.coreData.ExgQuote.PrevBestAskExg=m_ExgQuote.getShort(76);
		m_NxCoreMsg.coreData.ExgQuote.PrevBestBidExg=m_ExgQuote.getShort(78);
		m_NxCoreMsg.coreData.ExgQuote.BestAskCondition=m_ExgQuote.get(80);
		m_NxCoreMsg.coreData.ExgQuote.BestBidCondition=m_ExgQuote.get(81);
		m_NxCoreMsg.coreData.ExgQuote.BBOChangeFlags=m_ExgQuote.get(82);
		m_NxCoreMsg.coreData.ExgQuote.ClosingQuoteFlag=m_ExgQuote.get(83);
		//4 byte pad
		m_NxCoreMsg.MessageType=1;
	}
	private void populateMMQuote(String MarketMaker){
		m_NxCoreMsg.coreData.MMQuote.coreQuote.AskSize=m_ExgQuote.getInt(0);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.BidSize=m_ExgQuote.getInt(4);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.AskSizeChange=m_ExgQuote.getInt(8);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.BidSizeChange=m_ExgQuote.getInt(12);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.AskPrice=m_ExgQuote.getInt(16);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.BidPrice=m_ExgQuote.getInt(20);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.AskPriceChange=m_ExgQuote.getInt(24);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.BidPriceChange=m_ExgQuote.getInt(28);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.PriceType=m_ExgQuote.get(32);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.Refresh=m_ExgQuote.get(33);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.QuoteCondition=m_ExgQuote.get(34);
		m_NxCoreMsg.coreData.MMQuote.coreQuote.NasdaqBidTick=m_ExgQuote.get(35);
		
		m_NxCoreMsg.coreData.MMQuote.MarketMakerType=m_ExgQuote.get(48);
		m_NxCoreMsg.coreData.MMQuote.QuoteType=m_ExgQuote.get(49);
		//4 byte pad
		m_NxCoreMsg.coreData.MMQuote.pnxStringMarketMaker.String=MarketMaker;
		m_NxCoreMsg.MessageType=2;
	}
	private void populateTrade(){
		m_NxCoreMsg.coreData.Trade.Price=m_Trade.getInt(0);
		m_NxCoreMsg.coreData.Trade.PriceType=m_Trade.get(4);
		m_NxCoreMsg.coreData.Trade.PriceFlags=m_Trade.get(5);
		m_NxCoreMsg.coreData.Trade.TradeCondition=m_Trade.get(6) & 0xff;
		m_NxCoreMsg.coreData.Trade.ConditionFlags=m_Trade.get(7);
		m_NxCoreMsg.coreData.Trade.VolumeType=m_Trade.get(8);
		m_NxCoreMsg.coreData.Trade.BATECode=m_Trade.get(9);
//2 byte pad
		m_NxCoreMsg.coreData.Trade.Size=m_Trade.getInt(12);
		m_NxCoreMsg.coreData.Trade.ExgSequence=m_Trade.getInt(16);
		m_NxCoreMsg.coreData.Trade.RecordsBack=m_Trade.getInt(20);
		
		m_NxCoreMsg.coreData.Trade.nxAnalysis.FilterThreshold=m_Trade.getInt(24);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.Filtered=m_Trade.get(28);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.FilterLevel=m_Trade.get(29);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.SigHiLoType=m_Trade.get(30);
		//1 byte pad
		m_NxCoreMsg.coreData.Trade.nxAnalysis.SigHiLoSeconds=m_Trade.getInt(32);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.QteMatchDistanceRGN=m_Trade.getInt(36);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.QteMatchDistanceBBO=m_Trade.getInt(40);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.QteMatchTypeRGN=m_Trade.get(44);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.QteMatchTypeBBO=m_Trade.get(45);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.QteMatchFlagsRGN=m_Trade.get(46);
		m_NxCoreMsg.coreData.Trade.nxAnalysis.QteMatchFlagsBBO=m_Trade.get(47);
		m_NxCoreMsg.coreData.Trade.TotalVolume=m_Trade.getLong(48);
		m_NxCoreMsg.coreData.Trade.TickVolume=m_Trade.getInt(56);
		m_NxCoreMsg.coreData.Trade.Open=m_Trade.getInt(60);
		m_NxCoreMsg.coreData.Trade.High=m_Trade.getInt(64);
		m_NxCoreMsg.coreData.Trade.Low=m_Trade.getInt(68);
		m_NxCoreMsg.coreData.Trade.Last=m_Trade.getInt(72);
		m_NxCoreMsg.coreData.Trade.Tick=m_Trade.getInt(76);
		m_NxCoreMsg.coreData.Trade.NetChange=m_Trade.getInt(80);
		m_NxCoreMsg.coreData.Trade.ExtTradeConditions[0] = m_Trade.get(84) & 0xff;
		m_NxCoreMsg.coreData.Trade.ExtTradeConditions[1] = m_Trade.get(85) & 0xff;
		m_NxCoreMsg.coreData.Trade.ExtTradeConditions[2] = m_Trade.get(86) & 0xff;
		m_NxCoreMsg.coreData.Trade.ExtTradeConditions[3] = m_Trade.get(87) & 0xff;
		//4 byte pad
		m_NxCoreMsg.MessageType=3;
	}
	private void populateCategory(int categoryAtom, int NFields, String CategoryName, String[] FieldNames, String[] FieldsInfo, String[] DataStrings){
		m_NxCoreMsg.coreData.Category.pnxStringCategory.String=CategoryName;
		m_NxCoreMsg.coreData.Category.pnxStringCategory.Atom=categoryAtom;
		m_NxCoreMsg.coreData.Category.NFields=NFields;
		
		for(int i=0;i<NFields;i++){
			m_NxCoreMsg.coreData.Category.pnxFields[i].FieldName=FieldNames[i];
			m_NxCoreMsg.coreData.Category.pnxFields[i].FieldInfo=FieldsInfo[i];
			m_NxCoreMsg.coreData.Category.pnxFields[i].Set=m_Category.get(40*i+16);
			m_NxCoreMsg.coreData.Category.pnxFields[i].FieldType=m_Category.get(40*i+17);
			switch(m_NxCoreMsg.coreData.Category.pnxFields[i].FieldType){
			case 1:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.i64Bit=m_Category.getLong(40*i+24);
				break;
			case 2:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.i32Bit=m_Category.getInt(40*i+24);
				break;
			case 3:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.StringZ=DataStrings[i];
				break;
			case 4:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.Double=m_Category.getDouble(40*i+24);
				break;
			case 5:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxPrice.Price=m_Category.getInt(40*i+24);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxPrice.PriceType=m_Category.get(40*i+28);
				break;
			case 6:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.NDays=m_Category.getInt(40*i+24);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.Year=m_Category.getShort(40*i+28);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.Month=m_Category.get(40*i+30);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.Day=m_Category.get(40*i+31);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.DSTIndicator=m_Category.get(40*i+32);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.DayOfWeek=m_Category.get(40*i+33);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxDate.DayOfYear=m_Category.getShort(40*i+34);
				break;
			case 7:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.MsOfDay=m_Category.getInt(32);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.Millisecond=m_Category.getShort(36);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.Second=m_Category.get(38);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.Minute=m_Category.get(39);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.Hour=m_Category.get(40);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.TimeZone=m_Category.get(41);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.nxTime.MsResolution=m_Category.getShort(42);
				break;
			case 8:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.pnxString.String=DataStrings[i];
				break;
			case 9:
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.stringTableItem.ixTable=m_Category.getInt(40*i+24);
				m_NxCoreMsg.coreData.Category.pnxFields[i].data.stringTableItem.idString=m_Category.getInt(40*i+28);
				break;
			}
			
		}
		m_NxCoreMsg.MessageType=4;
	}
	private void populateSymbolChange(String oldSymbol){
		if(oldSymbol!=null){
			if(m_NxCoreMsg.coreData.SymbolChange.pnxsSymbolOld==null)
				m_NxCoreMsg.coreData.SymbolChange.pnxsSymbolOld = new NxString();
			m_NxCoreMsg.coreData.SymbolChange.pnxsSymbolOld.String = oldSymbol;
		}else
			m_NxCoreMsg.coreData.SymbolChange.pnxsSymbolOld.String = null;
		m_NxCoreMsg.coreData.SymbolChange.Status=m_SymbolChange.get(0);
		m_NxCoreMsg.coreData.SymbolChange.StatusChar=m_SymbolChange.get(1);
		//2 Byte Pad
		//8 Byte Pointer
		//8 Byte Pointer
		m_NxCoreMsg.coreData.SymbolChange.ListedExgOld=m_SymbolChange.getShort(20);
		//2 Byte Pad
		m_NxCoreMsg.MessageType=5;
	}
	private void populateSymbolChangeOptionHeader(String oldDateAndStrike,String oldUnderlying,String oldSeriesChain,String oldSpecialSettle) {
		if(oldDateAndStrike==null){
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld=null;
			return;
		}
		if(m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld==null)
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld = new NxOptionHdr();
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsDateAndStrike.String=oldDateAndStrike;
		if(oldUnderlying!=null){
			if(m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsUnderlying==null)
				m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsUnderlying = new NxString();
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsUnderlying.String=oldUnderlying;
		}
		else
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsUnderlying=null;
		if(oldSeriesChain!=null){
			if(m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSeriesChain == null)
				m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSeriesChain = new NxString();
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSeriesChain.String=oldSeriesChain;
		}
		else
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSeriesChain=null;
		if(oldSpecialSettle!=null){
			if(m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSpecialSettle==null)
				m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSpecialSettle = new NxString();
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSpecialSettle.String=oldSpecialSettle;
		}
		else
			m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.pnxsSpecialSettle=null;
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.exgUnderlying = m_SymbolChangeOptionHdr.getShort(32);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.contractUnit = m_SymbolChangeOptionHdr.getShort(34);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.strikePrice = m_SymbolChangeOptionHdr.getInt(36);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.strikePriceEst = m_SymbolChangeOptionHdr.getInt(40);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.PutCall = m_SymbolChangeOptionHdr.get(44);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.expirationCycle = m_SymbolChangeOptionHdr.get(45);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.oicStrikeAge = m_SymbolChangeOptionHdr.get(46);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxStrikeMatch = m_SymbolChangeOptionHdr.get(47);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.NDays = m_SymbolChangeOptionHdr.getInt(48);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.Year = m_SymbolChangeOptionHdr.getShort(52);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.Month = m_SymbolChangeOptionHdr.get(54);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.Day =  m_SymbolChangeOptionHdr.get(55);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.DSTIndicator = m_SymbolChangeOptionHdr.get(56);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.DayOfWeek = m_SymbolChangeOptionHdr.get(57);
		m_NxCoreMsg.coreData.SymbolChange.pnxOptionHdrOld.nxExpirationDate.DayOfYear = m_SymbolChangeOptionHdr.getShort(58);
	}
	private void populateSymbolSpin(){
		m_NxCoreMsg.coreData.SymbolSpin.SpinID = m_SymbolSpin.getInt(0);
		m_NxCoreMsg.coreData.SymbolSpin.IterLimit=m_SymbolSpin.getInt(4);
		m_NxCoreMsg.coreData.SymbolSpin.SubIterLimit=m_SymbolSpin.getInt(8);
		m_NxCoreMsg.coreData.SymbolSpin.IterCount=m_SymbolSpin.getInt(12);
		m_NxCoreMsg.coreData.SymbolSpin.SubIterCount=m_SymbolSpin.getInt(16);
		m_NxCoreMsg.MessageType=6;
	}
	private void populateUserSymbolSpin(String Symbol) {
		m_NxCoreSysSpin.UserData=m_Sys.getInt(0);
		m_NxCoreSysSpin.DLLVersion=m_Sys.getInt(4);
		m_NxCoreSysSpin.nxDate.NDays=m_Sys.getInt(8);
		m_NxCoreSysSpin.nxDate.Year=m_Sys.getShort(12);
		m_NxCoreSysSpin.nxDate.Month=m_Sys.get(14);
		m_NxCoreSysSpin.nxDate.Day=m_Sys.get(15);
		m_NxCoreSysSpin.nxDate.DSTIndicator=m_Sys.get(16);
		m_NxCoreSysSpin.nxDate.DayOfWeek=m_Sys.get(17);
		m_NxCoreSysSpin.nxDate.DayOfYear=m_Sys.getShort(18);
		//4 byte pad
		m_NxCoreSysSpin.nxTime.MsOfDay=m_Sys.getInt(24);
		m_NxCoreSysSpin.nxTime.Millisecond=m_Sys.getShort(28);
		m_NxCoreSysSpin.nxTime.Second=m_Sys.get(30);
		m_NxCoreSysSpin.nxTime.Minute=m_Sys.get(31);
		m_NxCoreSysSpin.nxTime.Hour=m_Sys.get(32);
		m_NxCoreSysSpin.nxTime.TimeZone=m_Sys.get(33);
		m_NxCoreSysSpin.nxTime.MsResolution=m_Sys.getShort(34);
		//4 byte pad
		m_NxCoreSysSpin.ClockUpdateInterval=m_Sys.getInt(40);
		m_NxCoreSysSpin.Status=m_Sys.getInt(44);
		m_NxCoreSysSpin.StatusData=m_Sys.getInt(48);
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.String=Symbol;
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.UserData1=m_SymbolUserData.getInt(0);
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.UserData2=m_SymbolUserData.getInt(4);
		m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.Atom=m_SymbolUserData.getShort(8);
		//8 byte pointer
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.NDays=m_coreHeader.getInt(16);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.Year=m_coreHeader.getShort(20);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.Month=m_coreHeader.get(22);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.Day=m_coreHeader.get(23);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.DSTIndicator=m_coreHeader.get(24);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.DayOfWeek=m_coreHeader.get(25);
		m_NxCoreMsgSpin.coreHeader.nxSessionDate.DayOfYear=m_coreHeader.getShort(26);
		//4 byte pad
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.MsOfDay=m_coreHeader.getInt(32);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Millisecond=m_coreHeader.getShort(36);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Second=m_coreHeader.get(38);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Minute=m_coreHeader.get(39);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.Hour=m_coreHeader.get(40);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.TimeZone=m_coreHeader.get(41);
		m_NxCoreMsgSpin.coreHeader.nxExgTimestamp.MsResolution=m_coreHeader.getShort(42);
		//4 byte pad
		m_NxCoreMsgSpin.coreHeader.ListedExg=m_coreHeader.getShort(48);
		m_NxCoreMsgSpin.coreHeader.ReportingExg=m_coreHeader.getShort(50);
		m_NxCoreMsgSpin.coreHeader.SessionID=m_coreHeader.get(52);
		//1 byte pad
		m_NxCoreMsgSpin.coreHeader.PermissionID=m_coreHeader.getShort(54);
		m_NxCoreMsgSpin.coreData.SymbolSpin.SpinID = m_SymbolSpin.getInt(0);
		m_NxCoreMsgSpin.coreData.SymbolSpin.IterLimit=m_SymbolSpin.getInt(4);
		m_NxCoreMsgSpin.coreData.SymbolSpin.SubIterLimit=m_SymbolSpin.getInt(8);
		m_NxCoreMsgSpin.coreData.SymbolSpin.IterCount=m_SymbolSpin.getInt(12);
		m_NxCoreMsgSpin.coreData.SymbolSpin.SubIterCount=m_SymbolSpin.getInt(16);
		m_NxCoreMsgSpin.MessageType=6;
	}
	private void saveSymbolUserData(){
		m_SymbolUserData.putInt(0,m_NxCoreMsg.coreHeader.pnxStringSymbol.UserData1);
		m_SymbolUserData.putInt(4,m_NxCoreMsg.coreHeader.pnxStringSymbol.UserData2);
	}
	private void saveOptionUserData(){
		if(m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike!=null) {
			m_ContractUserData.putInt(0,m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData1);
			m_ContractUserData.putInt(4,m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData2);
		}
		if(m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying!=null) {
			m_UnderlyingUserData.putInt(0,m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData1);
			m_UnderlyingUserData.putInt(4,m_NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData2);
		}
	}
	private void saveSymbolUserDataSpin(){
		m_SymbolUserData.putInt(0,m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.UserData1);
		m_SymbolUserData.putInt(4,m_NxCoreMsgSpin.coreHeader.pnxStringSymbol.UserData2);
	}
	private void saveOptionUserDataSpin(){
		if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike!=null) {
			m_ContractUserData.putInt(0,m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData1);
			m_ContractUserData.putInt(4,m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData2);
		}
		if(m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying!=null) {
			m_UnderlyingUserData.putInt(0,m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData1);
			m_UnderlyingUserData.putInt(4,m_NxCoreMsgSpin.coreHeader.pnxOptionHdr.pnxsUnderlying.UserData2);
		}
	}
	protected int cbStatus(String[] strings){
		m_NxCoreSys.UserData=m_Sys.getInt(0);
		m_NxCoreSys.DLLVersion=m_Sys.getInt(4);
		m_NxCoreSys.nxDate.NDays=m_Sys.getInt(8);
		m_NxCoreSys.nxDate.Year=m_Sys.getShort(12);
		m_NxCoreSys.nxDate.Month=m_Sys.get(14);
		m_NxCoreSys.nxDate.Day=m_Sys.get(15);
		m_NxCoreSys.nxDate.DSTIndicator=m_Sys.get(16);
		m_NxCoreSys.nxDate.DayOfWeek=m_Sys.get(17);
		m_NxCoreSys.nxDate.DayOfYear=m_Sys.getShort(18);
		//4 byte pad
		m_NxCoreSys.nxTime.MsOfDay=m_Sys.getInt(24);
		m_NxCoreSys.nxTime.Millisecond=m_Sys.getShort(28);
		m_NxCoreSys.nxTime.Second=m_Sys.get(30);
		m_NxCoreSys.nxTime.Minute=m_Sys.get(31);
		m_NxCoreSys.nxTime.Hour=m_Sys.get(32);
		m_NxCoreSys.nxTime.TimeZone=m_Sys.get(33);
		m_NxCoreSys.nxTime.MsResolution=m_Sys.getShort(34);
		//4 byte pad
		m_NxCoreSys.ClockUpdateInterval=m_Sys.getInt(40);
		m_NxCoreSys.Status=m_Sys.getInt(44);
		m_NxCoreSys.StatusData=m_Sys.getInt(48);
		//8 byte pointer
		//8 byte pointer
		//8 byte pointer
		//8 byte pointer
		//8 byte pointer
		//8 byte pointer
		m_NxCoreMsg.MessageType=0;
		if(GetMajorVersion(m_NxCoreSys.DLLVersion)>=3&&m_NxCoreMsg.pnxCore3Ext==null) {
			m_NxCoreMsg.pnxCore3Ext=new NxCore3Ext();
			m_NxCoreMsg.pnxCore3Ext.MicrosOfDaySRC=0;
			m_NxCoreMsg.pnxCore3Ext.MicrosOfDayPTP1=0;
			m_NxCoreMsg.pnxCore3Ext.MicrosOfDayPTP2=0;
		}else if(GetMajorVersion(m_NxCoreSys.DLLVersion)<3&&m_NxCoreMsg.pnxCore3Ext!=null) {
			m_NxCoreMsg.pnxCore3Ext=null;
		}
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		if(m_Sys.getInt(0)!=m_NxCoreSys.UserData)
			SetSystemUserData(m_NxCoreSys.UserData);
		return ret;
	}

	protected int cbExgQuoteEquity(String symbol){
		populateCoreHeader(symbol);
		populateExgQuote();
		m_NxCoreMsg.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		return ret;
	}
	protected int cbExgQuoteOption(String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle){
		populateCoreHeader(symbol);
		populateExgQuote();
		populateOptionHeader(dateAndStrike, underlying, seriesChain, specialSettle);
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		saveOptionUserData();
		return ret;
	}
	protected int cbMMQuoteEquity(String MarketMaker,String symbol){
		populateCoreHeader(symbol);
		populateMMQuote(MarketMaker);
		m_NxCoreMsg.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		return ret;
	}
	protected int cbMMQuoteOption(String MarketMaker,String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle){
		populateCoreHeader(symbol);
		populateMMQuote(MarketMaker);
		populateOptionHeader(dateAndStrike, underlying, seriesChain, specialSettle);
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		saveOptionUserData();
		return ret;
	}
	protected int cbTradeEquity(String symbol){
		populateCoreHeader(symbol);
		populateTrade();
		m_NxCoreMsg.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		return ret;
	}
	protected int cbTradeOption(String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle){
		populateCoreHeader(symbol);
		populateTrade();
		populateOptionHeader(dateAndStrike, underlying, seriesChain, specialSettle);
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		saveOptionUserData();
		return ret;
	}
	protected int cbCategoryEquity(int categoryAtom, int NFields, String CategoryName,String[] FieldNames,String[] FieldsInfo,String[] DataStrings,String symbol){
		populateCoreHeader(symbol);
		populateCategory(categoryAtom, NFields, CategoryName, FieldNames, FieldsInfo, DataStrings);
		m_NxCoreMsg.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		return ret;
	}
	protected int cbCategoryOption(int categoryAtom, int NFields, String CategoryName,String[] FieldNames,String[] FieldsInfo,String[] DataStrings,String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle){
		populateCoreHeader(symbol);
		populateCategory(categoryAtom, NFields, CategoryName, FieldNames, FieldsInfo, DataStrings);
		populateOptionHeader(dateAndStrike, underlying, seriesChain, specialSettle);
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		saveOptionUserData();
		return ret;
	}
	protected int cbSymbolChangeEquity(String oldSymbol,String symbol){
		populateCoreHeader(symbol);
		populateSymbolChange(oldSymbol);
		m_NxCoreMsg.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		return ret;
	}
	protected int cbSymbolChangeOption(String oldSymbol,String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle,String oldDateAndStrike,String oldUnderlying,String oldSeriesChain,String oldSpecialSettle){
		populateCoreHeader(symbol);
		populateSymbolChange(oldSymbol);
		populateOptionHeader(dateAndStrike, underlying, seriesChain, specialSettle);
		populateSymbolChangeOptionHeader(oldDateAndStrike, oldUnderlying, oldSeriesChain, oldSpecialSettle);
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		saveOptionUserData();
		return ret;
	}
	protected int cbSymbolSpinEquity(String symbol){
		populateCoreHeader(symbol);
		populateSymbolSpin();
		m_NxCoreMsg.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		return ret;
	}
	protected int cbSymbolSpinOption(String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle){
		populateCoreHeader(symbol);
		populateSymbolSpin();
		populateOptionHeader(dateAndStrike, underlying, seriesChain, specialSettle);
		int ret = 0;
		try {
			ret = OnNxCoreCallback(m_NxCoreSys, m_NxCoreMsg);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserData();
		saveOptionUserData();
		return ret;
	}

	protected int cbUserSymbolSpinEquity(String symbol){

		populateUserSymbolSpin(symbol);
		populateSymbolSpin();
		m_NxCoreMsgSpin.coreHeader.pnxOptionHdr=null;
		int ret = 0;
		try {
			ret = OnUserSymbolSpin(m_NxCoreSysSpin, m_NxCoreMsgSpin);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserDataSpin();
		return ret;
	}
	
	protected int cbUserSymbolSpinOption(String symbol,String dateAndStrike,String underlying,String seriesChain,String specialSettle){
		populateUserSymbolSpin(symbol);
		populateOptionHeaderSpin(dateAndStrike, underlying, seriesChain, specialSettle);
		int ret = 0;
		try {
			ret = OnUserSymbolSpin(m_NxCoreSysSpin, m_NxCoreMsgSpin);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		saveSymbolUserDataSpin();
		saveOptionUserDataSpin();
		return ret;
	}
	protected int cbTapeList(String PathnameStrZ,String PermSet){
		m_NxTF.PathnameStrZ=PathnameStrZ;
		m_NxTF.PathnameLen=m_TapeFileInfo.getInt(260);
		m_NxTF.FilenameLen=m_TapeFileInfo.getInt(264);
		m_NxTF.FileAttributes=m_TapeFileInfo.getInt(268);
		m_NxTF.FileSize=m_TapeFileInfo.getLong(272);
		m_NxTF.TapeDate.NDays=m_TapeFileInfo.getInt(280);
		m_NxTF.TapeDate.Year=m_TapeFileInfo.getShort(284);
		m_NxTF.TapeDate.Month=m_TapeFileInfo.get(286);
		m_NxTF.TapeDate.Day=m_TapeFileInfo.get(287);
		m_NxTF.TapeDate.DSTIndicator=m_TapeFileInfo.get(288);
		m_NxTF.TapeDate.DayOfWeek=m_TapeFileInfo.get(289);
		m_NxTF.TapeDate.DayOfYear=m_TapeFileInfo.getShort(290);
		m_NxTF.PermSet=PermSet;
		int ret = 0;
		try {
			ret = OnUserListTapes(m_NxTF);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		return ret;
	}
	protected int cbAPIList(String PathnameStrZ){
		m_NxAF.PathnameStrZ=PathnameStrZ;
		m_NxAF.PathnameLen=m_APIDLLFileInfo.getInt(260);
		m_NxAF.FilenameLen=m_APIDLLFileInfo.getInt(264);
		m_NxAF.FileAttributes=m_APIDLLFileInfo.getInt(268);
		m_NxAF.FileSize=m_APIDLLFileInfo.getLong(272);
		m_NxAF.BuildDate.NDays=m_APIDLLFileInfo.getInt(280);
		m_NxAF.BuildDate.Year=m_APIDLLFileInfo.getShort(284);
		m_NxAF.BuildDate.Month=m_APIDLLFileInfo.get(286);
		m_NxAF.BuildDate.Day=m_APIDLLFileInfo.get(287);
		m_NxAF.BuildDate.DSTIndicator=m_APIDLLFileInfo.get(288);
		m_NxAF.BuildDate.DayOfWeek=m_APIDLLFileInfo.get(289);
		m_NxAF.BuildDate.DayOfYear=m_APIDLLFileInfo.getShort(290);
		m_NxAF.BuildTime.MsOfDay=m_APIDLLFileInfo.getInt(296);
		m_NxAF.BuildTime.Millisecond=m_APIDLLFileInfo.getShort(300);
		m_NxAF.BuildTime.Second=m_APIDLLFileInfo.get(302);
		m_NxAF.BuildTime.Minute=m_APIDLLFileInfo.get(303);
		m_NxAF.BuildTime.Hour=m_APIDLLFileInfo.get(304);
		m_NxAF.BuildTime.TimeZone=m_APIDLLFileInfo.get(305);
		m_NxAF.BuildTime.MsResolution=m_APIDLLFileInfo.getShort(306);
		m_NxAF.verMajor=m_APIDLLFileInfo.get(312);
		m_NxAF.verMinor=m_APIDLLFileInfo.get(313);
		m_NxAF.verBuild=m_APIDLLFileInfo.getShort(314);
		int ret = 0;
		try {
			ret = OnUserListAPI(m_NxAF);
		}
		catch (Exception e) {
            e.printStackTrace();
        }
		return ret;
	}
	private native void SetSystemUserData(int userdata);
	private native int NativeGetExgQuotes(int exg,String symbol);
	private native String[] NativeGetMMQuotes(int ReportingExg,int exg,String symbol);
	private native int NativeGetLastTrade(int exg,String symbol);
	private native void NativeDateFromNDays(int NDays);
	private native void NativeDateFromYMD(int Year, int Month, int Day);
	//int GetStateData(char* pBuffer,int bufferSize,int stateType,int param1,int param2,NxString* pnxsSymOrCtc)
	//int SignalWaitEvent(const NxCoreSystem* pNxCoreSystem,bool bAlwaysSignal)
	//NxCoreCallback SetCallback(NxCoreCallback callbackFunction)
}