import NxCore
import multiprocessing

#path to the nx2 file, or leave blank for real-time using NxCore Access
filepath = ""

def OnNxCoreStatus(NxCoreSys, NxCoreMsg):
	if NxCoreSys.ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE and NxCoreSys.nxTime.Hour < 24:
		print("{:02d}/{:02d}/{:04d} {:02d}:{:02d}:{:02d}".format(NxCoreSys.nxDate.Month, NxCoreSys.nxDate.Day, NxCoreSys.nxDate.Year, NxCoreSys.nxTime.Hour, NxCoreSys.nxTime.Minute, NxCoreSys.nxTime.Second))
	if NxCoreSys.Status == NxCore.NxCORESTATUS_RUNNING:
		return
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_COMPLETE:
		print("NxCore Complete Message.")
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_INITIALIZING:
		print("NxCore Initialize Message.")
		print("API version is v{}.{}.{}".format(NxCore.GetMajorVersion(NxCoreSys.DLLVersion), NxCore.GetMinorVersion(NxCoreSys.DLLVersion), NxCore.GetBuildVersion(NxCoreSys.DLLVersion)))
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		print("NxCore Wait For Access.")
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_ERROR:
		print("NxCore Error.")
	return

def OnNxCoreTrade(NxCoreSys, NxCoreMsg):
	Trade = NxCoreMsg.coreData.Trade

	# Get the price and net change
	Price = NxCore.PriceToFloat(Trade.Price,Trade.PriceType)
	NetChange = NxCore.PriceToFloat(Trade.NetChange, Trade.PriceType)
	Size = Trade.Size

	print("Trade for Symbol: {}, Time: {:02d}:{:02d}:{:02d} Price: {:.2f} NetChg: {:.2f} Size: {} Exchg: {}".format(NxCoreMsg.coreHeader.pnxStringSymbol.String, NxCoreMsg.coreHeader.nxExgTimestamp.Hour, NxCoreMsg.coreHeader.nxExgTimestamp.Minute, NxCoreMsg.coreHeader.nxExgTimestamp.Second, Price, NetChange, Size, NxCoreMsg.coreHeader.ReportingExg))

def OnNxCoreExgQuote(NxCoreSys, NxCoreMsg):
	Quote = NxCoreMsg.coreData.ExgQuote
	coreQuote = Quote.coreQuote

	# Get the price and net change
	Bid = NxCore.PriceToFloat(coreQuote.BidPrice, coreQuote.PriceType)
	Ask = NxCore.PriceToFloat(coreQuote.AskPrice, coreQuote.PriceType)
	BidSize = coreQuote.BidSize
	AskSize = coreQuote.AskSize

	print("ExgQuote for Symbol: {}, Time: {:02d}:{:02d}:{:02d} Bid: {:.2f} Ask: {:.2f} BidSize: {} AskSize: {} Exchg: {}".format(NxCoreMsg.coreHeader.pnxStringSymbol.String, NxCoreMsg.coreHeader.nxExgTimestamp.Hour, NxCoreMsg.coreHeader.nxExgTimestamp.Minute, NxCoreMsg.coreHeader.nxExgTimestamp.Second, Bid, Ask, BidSize, AskSize, NxCoreMsg.coreHeader.ReportingExg))

def OnNxCoreMMQuote(NxCoreSys,NxCoreMsg):
	Quote = NxCoreMsg.coreData.MMQuote
	coreQuote = Quote.coreQuote

	# Get the price and net change
	Bid = NxCore.PriceToFloat(coreQuote.BidPrice, coreQuote.PriceType)
	Ask = NxCore.PriceToFloat(coreQuote.AskPrice, coreQuote.PriceType)
	BidSize = coreQuote.BidSize
	AskSize = coreQuote.AskSize

	print("MMQuote for Symbol: {}, MarketMaker: {} Time: {:02d}:{:02d}:{:02d} Bid: {:.2f} Ask: {:.2f} BidSize: {} AskSize: {} Exchg: {}".format(NxCoreMsg.coreHeader.pnxStringSymbol.String, Quote.pnxStringMarketMaker.String, NxCoreMsg.coreHeader.nxExgTimestamp.Hour, NxCoreMsg.coreHeader.nxExgTimestamp.Minute, NxCoreMsg.coreHeader.nxExgTimestamp.Second, Bid, Ask, BidSize, AskSize, NxCoreMsg.coreHeader.ReportingExg))

def OnNxCoreCallback(NxCoreSys, NxCoreMsg):
	if NxCoreMsg.MessageType == NxCore.NxMSG_STATUS:
		OnNxCoreStatus(NxCoreSys, NxCoreMsg)
	elif NxCoreMsg.MessageType == NxCore.NxMSG_TRADE:
		OnNxCoreTrade(NxCoreSys, NxCoreMsg)
	elif NxCoreMsg.MessageType == NxCore.NxMSG_EXGQUOTE:
		OnNxCoreExgQuote(NxCoreSys, NxCoreMsg)
	elif NxCoreMsg.MessageType == NxCore.NxMSG_MMQUOTE:
		OnNxCoreMMQuote(NxCoreSys, NxCoreMsg)
	return NxCore.NxCALLBACKRETURN_CONTINUE

def NxCoreThread(threadID):
	if threadID == 1:
		NxCore.FilterSymbols(["eSPY"])
	if threadID == 2:
		NxCore.FilterSymbols(["eQQQ"])
	if threadID == 3:
		NxCore.FilterSymbols(["eVTI"])
	if NxCore.LoadNxCore("../NxCoreAPI64.dll"):
		NxCore.ProcessTape(filepath, 0, NxCore.NxCF_EXCLUDE_OPRA, 0, OnNxCoreCallback)
	else:
		print("Failed to load NxCore Library")

if __name__ == "__main__":
	print("NxCore SampleApp05 Start.")
	thread1 = multiprocessing.Process(target=NxCoreThread, args=(1, ))
	thread2 = multiprocessing.Process(target=NxCoreThread, args=(2, ))
	thread3 = multiprocessing.Process(target=NxCoreThread, args=(3, ))
	thread1.start()
	thread2.start()
	thread3.start()
	thread1.join()
	thread2.join()
	thread3.join()
	print("NxCore SampleApp05 Stop.")