import NxCore
#NOTE: In this sample, we retrieve State    
#      items when NxCore delivers real time 
#      Trade, Quote and MMQuote messages.   
#      This is NOT the most effective way   
#      to get the streaming data (see       
#      Sample4). However it does effectively
#      demonstrate retrieving the State for 
#      any symbol.                          

#path to the nx2 file, or leave blank for real-time using NxCore Access
filepath = ""

def OnNxCoreStatus(NxCoreSys, NxCoreMsg):
	if NxCoreSys.ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE and NxCoreSys.nxTime.Hour < 24:
		print("{:02d}/{:02d}/{:04d} {:02d}:{:02d}:{:02d}".format(NxCoreSys.nxDate.Month, NxCoreSys.nxDate.Day, NxCoreSys.nxDate.Year, NxCoreSys.nxTime.Hour, NxCoreSys.nxTime.Minute, NxCoreSys.nxTime.Second))
	if NxCoreSys.Status == NxCore.NxCORESTATUS_RUNNING:
		return
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_COMPLETE:
		print("NxCore Complete Message.")
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_INITIALIZING:
		print("NxCore Initialize Message.")
		print("API version is v{}.{}.{}".format(NxCore.GetMajorVersion(NxCoreSys.DLLVersion), NxCore.GetMinorVersion(NxCoreSys.DLLVersion), NxCore.GetBuildVersion(NxCoreSys.DLLVersion)))
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		print("NxCore Wait For Access.")
	elif NxCoreSys.Status == NxCore.NxCORESTATUS_ERROR:
		print("NxCore Error.")
	return

def OnNxCoreTrade(NxCoreSys, NxCoreMsg):
	if NxCoreMsg.coreHeader.pnxOptionHdr:
		
		TradeState = NxCore.StateGetLastTrade(NxCoreMsg.coreHeader.ListedExg, NxCoreMsg.coreHeader.pnxStringSymbol.String, NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.NDays, NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice, NxCoreMsg.coreHeader.pnxOptionHdr.PutCall)
	else:
		TradeState = NxCore.StateGetLastTrade(NxCoreMsg.coreHeader.ListedExg, NxCoreMsg.coreHeader.pnxStringSymbol.String)
	if TradeState:
		print("StateLastTrade for {}, Open: {:.2f} High: {:.2f} Low: {:.2f} Last: {:.2f} Price: {:.2f} NetChg: {:.2f} TotalVol: {}".format(NxCoreMsg.coreHeader.pnxStringSymbol.String, NxCore.PriceToFloat(TradeState.Open, TradeState.PriceType), NxCore.PriceToFloat(TradeState.High, TradeState.PriceType), NxCore.PriceToFloat(TradeState.Low, TradeState.PriceType), NxCore.PriceToFloat(TradeState.Last, TradeState.PriceType), NxCore.PriceToFloat(TradeState.Price, TradeState.PriceType), NxCore.PriceToFloat(TradeState.NetChange, TradeState.PriceType), TradeState.TotalVolume))

def OnNxCoreExgQuote(NxCoreSys, NxCoreMsg):
	if NxCoreMsg.coreHeader.pnxOptionHdr:
		QuoteState = NxCore.StateGetExgQuotes(NxCoreMsg.coreHeader.ListedExg, NxCoreMsg.coreHeader.pnxStringSymbol.String, NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.NDays, NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice, NxCoreMsg.coreHeader.pnxOptionHdr.PutCall)
	else:
		QuoteState = NxCore.StateGetExgQuotes(NxCoreMsg.coreHeader.ListedExg, NxCoreMsg.coreHeader.pnxStringSymbol.String)
	if QuoteState:
		print("StateExgQuotes for {} BestBidExg: {}  BestAskExg: {}".format(NxCoreMsg.coreHeader.pnxStringSymbol.String, QuoteState.BestBidExg,QuoteState.BestAskExg))
		print(QuoteState.StateQuoteCount)
		for i in range(0,QuoteState.StateQuoteCount):
			Bid = NxCore.PriceToFloat(QuoteState.StateExgQuotes[i].BidPrice, QuoteState.PriceType)
			Ask = NxCore.PriceToFloat(QuoteState.StateExgQuotes[i].AskPrice, QuoteState.PriceType)
			BidSize = QuoteState.StateExgQuotes[i].BidSize
			AskSize = QuoteState.StateExgQuotes[i].AskSize
			ReportingExg = QuoteState.StateExgQuotes[i].ReportingExg;
			Condition = QuoteState.StateExgQuotes[i].QuoteCondition;
			print("Exg: {}, Bid: {:.2f} Ask: {:.2f} BidSize: {} AskSize: {} Condition: {}".format(ReportingExg, Bid, Ask, BidSize, AskSize, Condition))


def OnNxCoreMMQuote(NxCoreSys, NxCoreMsg):
	QuoteState=NxCore.StateGetMMQuotes(NxCoreMsg.coreHeader.ReportingExg,NxCoreMsg.coreHeader.ListedExg, NxCoreMsg.coreHeader.pnxStringSymbol.String)
	if QuoteState:
		print("StateMMQuotes for {} Count= {}".format(NxCoreMsg.coreHeader.pnxStringSymbol.String, QuoteState.StateQuoteCount))
		for i in range(0, QuoteState.StateQuoteCount):
			Bid = NxCore.PriceToFloat(QuoteState.StateMMQuotes[i].BidPrice, QuoteState.PriceType)
			Ask = NxCore.PriceToFloat(QuoteState.StateMMQuotes[i].AskPrice, QuoteState.PriceType)
			BidSize = QuoteState.StateMMQuotes[i].BidSize
			AskSize = QuoteState.StateMMQuotes[i].AskSize
			print("Market Maker:  {}, Bid: {:.2f} Ask: {:.2f} BidSize: {} AskSize: {}".format(QuoteState.StateMMQuotes[i].pnxStringMarketMaker.String, Bid, Ask, BidSize, AskSize))

def OnNxCoreCallback(NxCoreSys,NxCoreMsg):
	if NxCoreMsg.MessageType == NxCore.NxMSG_STATUS:
		OnNxCoreStatus(NxCoreSys, NxCoreMsg)
	elif NxCoreMsg.MessageType == NxCore.NxMSG_TRADE:
		OnNxCoreTrade(NxCoreSys, NxCoreMsg)
	elif NxCoreMsg.MessageType == NxCore.NxMSG_EXGQUOTE:
		OnNxCoreExgQuote(NxCoreSys, NxCoreMsg)
	elif NxCoreMsg.MessageType == NxCore.NxMSG_MMQUOTE:
		OnNxCoreMMQuote(NxCoreSys, NxCoreMsg)
	return NxCore.NxCALLBACKRETURN_CONTINUE

print("NxCore SampleApp06 Start.")
#NxCore.FilterSymbols(["eAAPL"])
if NxCore.LoadNxCore("../NxCoreAPI64.dll"):
	NxCore.ProcessTape(filepath, 0, 0, 0, OnNxCoreCallback)
else:
	print("Failed to load NxCore Library")