#ifndef NX_CSUPPORT_C 
#define NX_CSUPPORT_C 
 
#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 

#if !defined(_WIN32) && !defined(_WIN64)
#include <stdbool.h>
#include <dlfcn.h> 
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#define NxAPI_MODULE void*
#define NxGetAPISymbol dlsym
#define NxAPIClose  dlclose
static void* NxAPIOpen(const char* library) {
	char filename[260];
	memset(filename, 0, 260);
	if(readlink(library, filename, 260) < 0)
		strcpy(filename, library);
	return dlopen(filename, RTLD_LAZY | RTLD_DEEPBIND);
}
#else
#include "windows.h"
//#defines for C compatibility in Visual Studio 
#define false   0
#define true    1
#define bool	int

#define NxAPI_MODULE HMODULE
#define NxGetAPISymbol GetProcAddress
#define NxAPIClose FreeLibrary
#define NxAPIOpen LoadLibrary
#endif

/***************************************************************************** 
 * Makes NX compatible with standard C and includes Nanex-provided headers.  * 
 *****************************************************************************/ 
 
typedef struct NxDate NxDate;
typedef struct NxTime NxTime;
typedef struct NxString NxString;
typedef struct NxAccessStatus NxAccessStatus;
typedef struct NxCoreSystem NxCoreSystem;
typedef struct NxCoreQuote NxCoreQuote;
typedef struct NxCoreExgQuote NxCoreExgQuote;
typedef struct NxCoreMMQuote NxCoreMMQuote;
typedef struct NxCTAnalysis NxCTAnalysis;
typedef struct NxCoreTrade NxCoreTrade;
typedef struct NxPrice NxPrice;
typedef struct StringTableItem StringTableItem;
typedef struct NxCategoryField NxCategoryField;
typedef struct NxCoreCategory NxCoreCategory;
typedef struct NxOptionHdr NxOptionHdr;
typedef struct NxCoreSymbolChange NxCoreSymbolChange;
typedef struct NxCoreSymbolSpin NxCoreSymbolSpin;
typedef union NxCoreData NxCoreData;
typedef struct NxCoreHeader NxCoreHeader;
typedef struct NxCoreMessage NxCoreMessage;
typedef struct NxCoreStateExgQuote NxCoreStateExgQuote;
typedef struct NxCoreStateMMQuote NxCoreStateMMQuote;
typedef struct NxCoreStateExgQuotes NxCoreStateExgQuotes;
typedef struct NxCoreStateMMQuotes NxCoreStateMMQuotes;
typedef struct NxCoreStateTrade NxCoreStateTrade;
typedef struct NxCoreStateOHLCVTrade NxCoreStateOHLCVTrade;
typedef struct NxCoreTapeFile NxCoreTapeFile;
typedef struct NxCoreAPIDLLFile NxCoreAPIDLLFile;
typedef struct NxCore3Ext NxCore3Ext;

#include "NxCoreAPI.h"

NxCoreAPIVersion pfNxCoreAPIVersion;
NxCoreProcessTape pfNxCoreProcessTape;
NxCoreListTapes pfNxCoreListTapes;
NxCoreListTapesFolder pfNxCoreListTapesFolder;
NxCoreListAPIDLLs pfNxCoreListAPIDLLs;
NxCoreListAPIDLLsFolder pfNxCoreListAPIDLLsFolder;
NxCoreSpinSymbols pfNxCoreSpinSymbols;
NxCoreSaveState pfNxCoreSaveState;
NxCoreGetStateData pfNxCoreGetStateData;
NxCoreStateGetExgQuotes pfNxCoreStateGetExgQuotes;
NxCoreStateGetMMQuotes pfNxCoreStateGetMMQuotes;
NxCoreStateGetLastTrade pfNxCoreStateGetLastTrade;
NxCorePriceConvert pfNxCorePriceConvert;
NxCorePriceFormat pfNxCorePriceFormat;
NxCorePriceToDouble pfNxCorePriceToDouble;
NxCoreDateFromNDays pfNxCoreDateFromNDays;
NxCoreDateFromYMD pfNxCoreDateFromYMD;
NxCoreGetDefinedString pfNxCoreGetDefinedString;
NxCoreGetTapeName pfNxCoreGetTapeName;
NxCoreSignalWaitEvent pfNxCoreSignalWaitEvent;
NxCoreSetCallback pfNxCoreSetCallback;
NxCoreGetSymbolAtom32 pfGetSymbolAtom32;
NxCoreExcludeOpraExch pfExcludeOpraExch;//sets OPRA NBBO or OPRA Full exclusion int. See NxOPRAID* in NxCoreAPI.h for list.

// Standard dynamic loading support 
#define LoadNxCore   loadNxCore
#define UnloadNxCore unloadNxCore

static NxAPI_MODULE NxLib;

/**
 * Unloads the NxCoreAPI.dll/libnx.so if it was loaded
 */
static void unloadNxCore()
{ 
  if (NxLib) 
    NxAPIClose(NxLib);
} 

/**
 * Loads the NxCoreAPI.dll/libnx.so specified
 */
static unsigned char loadNxCore(char const *FileName)
{ 
  bool fret = false;
 
  unloadNxCore();
  if ((NxLib = NxAPIOpen(FileName)) != 0) {
    fret |= (pfNxCoreAPIVersion = (NxCoreAPIVersion)NxGetAPISymbol(NxLib, cszNxCoreAPIVersion)) == 0;
    fret |= (pfNxCoreProcessTape = (NxCoreProcessTape)NxGetAPISymbol(NxLib, cszNxCoreProcessTape)) == 0;
    fret |= (pfNxCoreListTapes = (NxCoreListTapes)NxGetAPISymbol(NxLib, cszNxCoreListTapes)) == 0;
    fret |= (pfNxCoreListAPIDLLs = (NxCoreListAPIDLLs)NxGetAPISymbol(NxLib, cszNxCoreListAPIDLLs)) == 0;
    fret |= (pfNxCoreSpinSymbols = (NxCoreSpinSymbols)NxGetAPISymbol(NxLib, cszNxCoreSpinSymbols)) == 0;
    fret |=	(pfNxCoreSaveState = (NxCoreSaveState)NxGetAPISymbol(NxLib, cszNxCoreSaveState)) == 0;
    fret |= (pfNxCoreStateGetExgQuotes = (NxCoreStateGetExgQuotes)NxGetAPISymbol(NxLib, cszNxCoreStateGetExgQuotes)) == 0;
    fret |= (pfNxCoreStateGetMMQuotes = (NxCoreStateGetMMQuotes)NxGetAPISymbol(NxLib, cszNxCoreStateGetMMQuotes)) == 0;
    fret |= (pfNxCoreStateGetLastTrade = (NxCoreStateGetLastTrade)NxGetAPISymbol(NxLib, cszNxCoreStateGetLastTrade)) == 0;
    fret |= (pfNxCoreGetDefinedString = (NxCoreGetDefinedString)NxGetAPISymbol(NxLib, cszNxCoreGetDefinedString)) == 0;
    fret |= (pfNxCoreGetTapeName = (NxCoreGetTapeName)NxGetAPISymbol(NxLib, cszNxCoreGetTapeName)) == 0;
    fret |= (pfNxCoreDateFromNDays = (NxCoreDateFromNDays)NxGetAPISymbol(NxLib, cszNxCoreDateFromNDays)) == 0;
    fret |= (pfNxCoreDateFromYMD = (NxCoreDateFromYMD)NxGetAPISymbol(NxLib, cszNxCoreDateFromYMD)) == 0;
    fret |= (pfNxCorePriceConvert = (NxCorePriceConvert)NxGetAPISymbol(NxLib, cszNxCorePriceConvert)) == 0;
    fret |= (pfNxCorePriceFormat = (NxCorePriceFormat)NxGetAPISymbol(NxLib, cszNxCorePriceFormat)) == 0;
    fret |= (pfNxCorePriceToDouble = (NxCorePriceToDouble)NxGetAPISymbol(NxLib, cszNxCorePriceToDouble)) == 0;

    if (!fret) { 
      fret |= (pfNxCoreSetCallback = (NxCoreSetCallback)NxGetAPISymbol(NxLib, cszNxCoreSetCallback)) == 0;
      fret |= (pfNxCoreGetStateData = (NxCoreGetStateData)NxGetAPISymbol(NxLib, cszNxCoreGetStateData)) == 0;
      fret |= (pfGetSymbolAtom32 = (NxCoreGetSymbolAtom32)NxGetAPISymbol(NxLib, cszNxCoreGetSymbolAtom32)) == 0;
	  fret |= (pfExcludeOpraExch = (NxCoreExcludeOpraExch)NxGetAPISymbol(NxLib, cszNxCoreExcludeOpraExch)) == 0;
	  fret |= (pfNxCoreListTapesFolder = (NxCoreListTapesFolder)NxGetAPISymbol(NxLib, cszNxCoreListTapesFolder)) == 0;
	  fret |= (pfNxCoreListAPIDLLsFolder = (NxCoreListAPIDLLsFolder)NxGetAPISymbol(NxLib, cszNxCoreListAPIDLLsFolder)) == 0;
	} else {
		NxAPIClose(NxLib);
    }

  } 
  return (NxLib ? 1 : 0);
} 

/**
 * Helper function that outputs human-readable values based on the return from processTape()
 */
static void processReturnValue(int returnValue) {
    switch(returnValue) {
        case NxAPIERR_NO_ERROR: // 0
            printf("Tape completed normally.\n");
            break;
        case NxAPIERR_USER_STOPPED: // 1
            printf("NxCALLBACKRETURN_STOP was returned from callback function.\n");
            break;
        case NxAPIERR_NOT_CALLBACK_THREAD: // -1
            printf("Function was called from thread other than callback thread.\n");
            break;
        case NxAPIERR_BAD_PARAMETERS: // -2
            printf("Function was called from thread other than callback thread.\n");
            break;
        case NxAPIERR_EXCEPTION: // -3
            printf("An exception occurred.\n");
            break;
        case NxAPIERR_OPEN_TAPE_FILE_FAILED: // -4
            printf("Could not open the file specified as parameter 1.\n");
            break;
        case NxAPIERR_INITIALIZE_MEMORY_FAILED: // -5
            printf("Could not initialize memory. This can occur when calling processTape with a saved state file and the memory address used to save the state has been used by another dll or within your process.\n");
            break;
        case NxAPIERR_NOLISTED_EXG: // -6
            printf("Symbol spin -- listed exchange does not exist in current tape\n");
            break;
         case NxAPIERR_NOSYMBOLS_FOR_LSTEXG: // -7
            printf("Symbol spin -- no symbols of the type (options/not options) for the exchange specified\n");
            break;
         case NxAPIERR_NOSESSION_FOR_SYMBOL: // -8
            printf("Symbol spin -- no session or session does not have data type\n");
            break;
         case NxAPIERR_NODATATYPE_FOR_SESSION: // -9
            printf("Symbol spin -- There's a session, but no trade/quote/mmquote data for session\n");
            break;
         case NxAPIERR_NODATA_FOR_REPEXG: // -10
            printf("Symbol spin -- MMQuotes. There is session data, but no entry for the specified rep. exg\n");
            break;
         case NxAPIERR_ZEROED_DATA_FOR_SESSION: // -11
            printf("Symbol spin -- there is a session, but data is all zero\n");
            break;
         case NxAPIERR_SAVE_STATE_IN_PROGRESS: // -12
            printf("Save state -- a state save is currently in progress\n");
            break;
         case NxAPIERR_NOT_SUPPORTED: // -13
            printf("General error\n");
            break;
         case NxAPIERR_INITALLOC_ERROR: // -14
            printf("Startup initial allocation error\n");
            break;
         case NxAPIERR_NSODALLOC_ERROR: // -15
            printf("Startup pNxSOD allocation error\n");
            break;
         default:
            printf("Unknown return value: (%d)",
                   returnValue);
            break;
    }
}


#endif // NX_CSUPPORT_C 
