////////////////////////////////////////////////////
//// SampleApp01.c                              ////
//// Sample App for NxCore in C                 ////
//// Authors: Jeffrey Donovan, Jeremy Truelove  ////
//// Date: 02-08-2017                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) Starting NxCore ProcessTape             ////
//// 2) Defining/using NxCore Callback Function ////
//// 3) Handling the NxCore Status Message      ////
////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		printf("SampleApp01 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}

	printf("NxCore C SampleApp01 Start.\n");
	if(LoadNxCore(argv[1])) {
        // If a tape filename was passed in command line argument,
        // call ProcessTape with that argument. Use control flags
        // to eliminate Level1 quotes, Level2 quotes and all OPRA
        // quotes.
		if(argv[2]){
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
	}
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp01 Stop.\n");
	exit(EXIT_SUCCESS);
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // If this is an NxCore Status message pass it to the status
  // message handler function.
  if (pNxCoreMsg->MessageType == NxMSG_STATUS)
         OnNxCoreStatus(pNxCoreSys,pNxCoreMsg);

  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}


//------------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
// If a Minute has elapsed print the NxCore system time.
// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
// a real time value

    if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
            (pNxCoreSys->nxTime.Hour < 24))
	{
		printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
			pNxCoreSys->nxDate.Month,
			pNxCoreSys->nxDate.Day,
			pNxCoreSys->nxDate.Year,
			pNxCoreSys->nxTime.Hour,
			pNxCoreSys->nxTime.Minute,
			pNxCoreSys->nxTime.Second);
	}

    // Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
      case NxCORESTATUS_COMPLETE:
        printf("NxCore Complete Message.\n");
        break;

      case NxCORESTATUS_INITIALIZING:
        printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
			NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
        break;

      case NxCORESTATUS_SYNCHRONIZING:
        printf("NxCore Synchronizing Message.\n");
        break;

      case NxCORESTATUS_WAITFORCOREACCESS:
        printf("NxCore Wait For Access.\n");
        break;

      case NxCORESTATUS_RESTARTING_TAPE:
        printf("NxCore Restart Tape Message.\n");
        break;

      case NxCORESTATUS_ERROR:
        printf("NxCore Error.\n");
        break;

      case NxCORESTATUS_RUNNING:
        break;

    }
}
