/////////////////////////////////////////////////////
//// SampleApp03.c                               ////
//// Sample App for NxCore in C                  ////
//// Authors: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 02-08-2017                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// 1) Handling the NxCore Category Mesasge     ////
/////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>    				// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreCategory(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

int main(int argc, char* argv[])
{
	int returnValue;
	if(argc < 2) {
		printf("SampleApp03 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}

	printf("NxCore C SampleApp03 Start.\n");
	if(LoadNxCore(argv[1])) {
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if(argv[2]){
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

		// Unload NxCore DLL
		UnloadNxCore();
	}
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp03 Stop.\n");
	exit(EXIT_SUCCESS);
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{

// Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
        // NxCore Status Message
        case NxMSG_STATUS:
             OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
                 break;

        // NxCore Symbol Spin Message
        case NxMSG_CATEGORY:
             OnNxCoreCategory(pNxCoreSys, pNxCoreMsg);
                 break;
          }


  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}

// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
// If a Minute has elapsed print the NxCore system time.
// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
// a real time value

	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
			(pNxCoreSys->nxTime.Hour < 24))
	{
		printf("NxCore Time: %02d/%02d/%d %02d:%02d:%02d\n",
				pNxCoreSys->nxDate.Month,
				pNxCoreSys->nxDate.Day,
				pNxCoreSys->nxDate.Year,
				pNxCoreSys->nxTime.Hour,
				pNxCoreSys->nxTime.Minute,
				pNxCoreSys->nxTime.Second);
	}

    // Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
      case NxCORESTATUS_COMPLETE:
        printf("NxCore Complete Message.\n");
        break;

      case NxCORESTATUS_INITIALIZING:
        printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
			NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
        break;

      case NxCORESTATUS_SYNCHRONIZING:
        printf("NxCore Synchronizing Message.\n");
        break;

      case NxCORESTATUS_WAITFORCOREACCESS:
        printf("NxCore Wait For Access.\n");
        break;

      case NxCORESTATUS_RESTARTING_TAPE:
        printf("NxCore Restart Tape Message.\n");
        break;

      case NxCORESTATUS_ERROR:
        printf("NxCore Error.\n");
        break;

      case NxCORESTATUS_RUNNING:
        break;

    }
}
// OnNxCoreCategory: Function to handle NxCore Category messages.
// In this example we only capture category 4 and category 13 messages.
//--------------------------------------------------------------
void OnNxCoreCategory(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  // Category field pointer
  NxCategoryField* pField;
  double High52, Low52;
  NxDate HighDate;
  NxDate LowDate;

  // Do something based on the category num
  switch (pNxCoreMsg->coreData.Category.pnxStringCategory->Atom)
  {
	// Category 4, Equity Symbol Information
	case 4:

		// Grab a pointer to the field
		pField = &pNxCoreMsg->coreData.Category.pnxFields[0];

		// Is field set?
		if (pField->Set == 1) {
			// Print the Company Name
			printf("Category 4 for Symbol: %s, Company Name= %s\n",
					pNxCoreMsg->coreHeader.pnxStringSymbol->String,
					pField->data.StringZ);
		}
		break;

		// Category 13, 52 Week High/Low
	case 13:

		High52 = Low52 = 0.0;

		memset(&HighDate, 0, sizeof(NxDate));
		memset(&LowDate, 0, sizeof(NxDate));

		// Grab a pointer to the 52 week high date
		pField = &pNxCoreMsg->coreData.Category.pnxFields[0];
		if (pField->Set == 1)
			memcpy(&HighDate, &pField->data.nxDate, sizeof(NxDate));
		// Grab a pointer to the 52 week low date
		pField = &pNxCoreMsg->coreData.Category.pnxFields[1];
		if (pField->Set == 1)
			memcpy(&LowDate, &pField->data.nxDate, sizeof(NxDate));

		// Grab a pointer to the 52 week high field
		pField = &pNxCoreMsg->coreData.Category.pnxFields[2];

		// Get the 52 week high
		if (pField->Set == 1)
			High52 = pfNxCorePriceToDouble(pField->data.nxPrice.Price,
					pField->data.nxPrice.PriceType);

		// Grab a pointer to the 52 week low field
		pField = &pNxCoreMsg->coreData.Category.pnxFields[3];

		// Get the 52 week low
		if (pField->Set == 1)
			Low52 = pfNxCorePriceToDouble(pField->data.nxPrice.Price,
					pField->data.nxPrice.PriceType);

		printf("Category 13 for Symbol: %s, %02d/%02d/%04d High52= %0.2f, %02d/%02d/%04d Low52= %0.2f\n",
				pNxCoreMsg->coreHeader.pnxStringSymbol->String,
				HighDate.Month,
				HighDate.Day,
				HighDate.Year,
				High52,
				LowDate.Month,
				LowDate.Day,
				LowDate.Year,
				Low52);

		break;
	}
}


