//////////////////////////////////////////////////////
//// SampleApp09.c                                ////
//// Sample App for NxCore in C                   ////
//// Authors: Jeffrey Donovan, Jeremy Truelove    ////
//// Date: 02-08-2017                             ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// Sending User Defined Data through the        ////
//// ProcessTape function, retrieving it in the   ////
//// NxCore Callback.                             ////
//////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>    				// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
//void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

NxDate gNxDates[3];

int main(int argc, char* argv[])
{
	int returnValue,dateIndex;

	if(argc < 2) {
		printf("SampleApp09 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}
	printf("NxCore C SampleApp09 Start.\n");

    // If we can load the NxCore library
	if (LoadNxCore(argv[1]))
    {
		dateIndex = 2;

		gNxDates[dateIndex].Day = 8;
		gNxDates[dateIndex].Month = 3;
		gNxDates[dateIndex].Year = 2020;

		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if (argv[2]) {
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, dateIndex, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, dateIndex, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
    }
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp09 Stop.\n");
	exit(EXIT_SUCCESS);
}

// The NXCore Callback Function 
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{   
  //Grab the data from UserData
  int dateIndex = pNxCoreSys->UserData;

  //print out custom data
  printf("UserData Date: %02d/%02d/%d\n",
		  gNxDates[dateIndex].Month,
		  gNxDates[dateIndex].Day,
		  gNxDates[dateIndex].Year);

  //stop processing the tape
  return NxCALLBACKRETURN_STOP;
}
