////////////////////////////////////////////////////
//// SampleApp1.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 12-21-09                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) Starting NxCore ProcessTape             ////
//// 2) Defining/using NxCore Callback Function ////
//// 3) Handling the NxCore Status Message      ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;


// Use the NxCoreAPI namespace
using NxCoreAPI;

namespace SampleApp1
{ 
  
  // The Program!
  class Program
  {
	
    // Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp1 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. Use control flags
	  // to eliminate Level1 quotes, Level2 quotes and all OPRA
	  // quotes.

	  if (args.Length > 0)
	  {
		NxCore.ProcessTape(args[0],
							 null,
							 (uint)(NxCore.NxCF_EXCLUDE_QUOTES | NxCore.NxCF_EXCLUDE_QUOTES2 | NxCore.NxCF_EXCLUDE_OPRA),
							 0,
							 Program.OnNxCoreCallback);		
	  }

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
	  {
		NxCore.ProcessTape("",
						   null,
						   (uint)(NxCore.NxCF_EXCLUDE_QUOTES | NxCore.NxCF_EXCLUDE_QUOTES2 | NxCore.NxCF_EXCLUDE_OPRA),
						   0,
						   Program.OnNxCoreCallback);
	  }

	  Console.WriteLine("NxCore CSharp SampleApp1 Stop.");
	}


	// OnNxCoreCallback: The NXCore Callback Function	
	//-----------------------------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;
	   
	    // If this is an NxCore Status message pass it to the status
	  // message handler function.
	  if (pNxCoreMsg->MessageType == NxCore.NxMSG_STATUS)
		OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
	  	  
	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}

	
	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	  
	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime=new DateTime(pNxCoreSys->nxDate.Year,
		                               pNxCoreSys->nxDate.Month,
									   pNxCoreSys->nxDate.Day,
									   pNxCoreSys->nxTime.Hour,
						               pNxCoreSys->nxTime.Minute,
						               pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);	
	  }

	  // Print the specific NxCore status message
	  switch( pNxCoreSys->Status ) 
      {
		case NxCore.NxCORESTATUS_COMPLETE:
		     Console.WriteLine("NxCore Complete Message.");		     
		     break;

 	    case NxCore.NxCORESTATUS_INITIALIZING:
		     Console.WriteLine("NxCore Initialize Message.");		   
		     break;

	    case NxCore.NxCORESTATUS_SYNCHRONIZING:
		     Console.WriteLine("NxCore Synchronizing Message.");
		     break;

	    case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		     Console.WriteLine("NxCore Wait For Access.");
		     break;

	    case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		     Console.WriteLine("NxCore Restart Tape Message.");
		     break;

	    case NxCore.NxCORESTATUS_ERROR:
		     Console.WriteLine("NxCore Error.");
		     break;

	    case NxCore.NxCORESTATUS_RUNNING:
		     break;
      }
	}		
  
  }
}
