////////////////////////////////////////////////////
//// SampleApp9.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 12-21-09                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Sending User Defined Data through the      ////
//// ProcessTape function, retrieving it in the ////
//// NxCore Callback.                           ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

// Use the NxCoreAPI namespace
using NxCoreAPI;

namespace SampleApp9
{ 
    
  // The Program!
  class Program
  {
    // Main Entry Point for app.	
    //--------------------------
    static void Main(string[] args)
	{
	  
	  Console.WriteLine("NxCore CSharp SampleApp1 Start.");

      uint MyData = 2;

      // If a tape filename was passed in command line argument,
      // call ProcessTape with that argument. Use control flags
      // to eliminate Level1 quotes, Level2 quotes and all OPRA
      // quotes.
      // Send in User Defined Data	
      if (args.Length > 0)
	  {	
		unsafe
		{
		  NxCore.ProcessTape(args[0],
							 null,
							 (uint)(NxCore.NxCF_EXCLUDE_QUOTES | NxCore.NxCF_EXCLUDE_QUOTES2 | NxCore.NxCF_EXCLUDE_OPRA),
							 (uint)MyData,
							 Program.OnNxCoreCallback);
		}
	  }

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
	  {
		unsafe
		{
		  NxCore.ProcessTape("",
							 null,
							 (uint)(NxCore.NxCF_EXCLUDE_QUOTES | NxCore.NxCF_EXCLUDE_QUOTES2 | NxCore.NxCF_EXCLUDE_OPRA),
							 (uint)MyData,
							 Program.OnNxCoreCallback);
		}
	  }

	  Console.WriteLine("NxCore CSharp SampleApp1 Stop.");
	}


	// OnNxCoreCallback: The NXCore Callback Function	
	//-----------------------------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  Console.WriteLine("");

	  // Print out custom data
	  Console.WriteLine("UserData: {0}", pNxCoreSys->UserData);

	  Console.WriteLine("");
	  Console.WriteLine("Press <Enter> to STOP.");
	  Console.ReadKey();
	  	  
	  // Stop The Tape
	  return (int)NxCore.NxCALLBACKRETURN_STOP;
	}
	
	
  }
}
