////////////////////////////////////////////////////
//// SampleApp11.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 12-22-09                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Handling NxCore Trade and Quote Messages   ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;


public struct JTime
{
  public byte Hour;
  public byte Minute;
  public byte Second;
  public short Millisecond;
  public short Microsecond;
  public int MsOfDay;
  public UInt64 MicrosOfDay;
};


namespace SampleApp11
{ 

  class Program
  {

	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp11 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null, 0, 0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null, 0, 0,
						   Program.OnNxCoreCallback);


	  Console.WriteLine("NxCore CSharp SampleApp11 Stop.");
	}


	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {
		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Trade Message
		case NxCore.NxMSG_TRADE:
		  OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Level1 Quote Message
		case NxCore.NxMSG_EXGQUOTE:
		  //OnNxCoreExgQuote(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Level2 Quote Message
		case NxCore.NxMSG_MMQUOTE:
		  //OnNxCoreMMQuote(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}
	
	
	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);
	  }
	  
	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		case NxCore.NxCORESTATUS_COMPLETE:
		     Console.WriteLine("NxCore Complete Message.");
		     break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		     Console.WriteLine("NxCore Initialize Message.");
		     break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		     Console.WriteLine("NxCore Synchronizing Message.");
		     break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		     Console.WriteLine("NxCore Wait For Access.");
		     break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		     Console.WriteLine("NxCore Restart Tape Message.");
		     break;

		case NxCore.NxCORESTATUS_ERROR:
		     Console.WriteLine("NxCore Error.");
		     break;

		case NxCore.NxCORESTATUS_RUNNING:
		     break;
	  }	  
	}

    static int JTime_MicroDiffFromMSofDay(UInt64 micros,int ms) 
    {
      return ((int)(micros - ((UInt64)ms * 1000)));
    }

    // Convert MicrosecondOfDay to JTime
    static unsafe void JTime_MicrosOfDayToTime(UInt64 MicrosOfDay,JTime* Time)
    {
       Time->MicrosOfDay=MicrosOfDay;
   
       Time->MsOfDay=(int)(MicrosOfDay/1000);   
       int Seconds=Time->MsOfDay/1000;
     
       Time->Hour       =(byte) (Seconds/(60*60));
       Time->Minute     =(byte) (Seconds/60%60);
       Time->Second     =(byte) (Seconds%60);
       Time->Millisecond=(short)(Time->MsOfDay % 1000);
       Time->Microsecond=(short) JTime_MicroDiffFromMSofDay(MicrosOfDay,Time->MsOfDay);         
    }

	// OnNxCoreTrade: Function to handle NxCore Trade messages.	
	//--------------------------------------------------------------
	static unsafe void OnNxCoreTrade(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // Get the symbol for category message
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);

      if (Symbol[0] == 'e')
      {
        // Assign a pointer to the Trade data
        NxCoreTrade* Trade = &pNxCoreMsg->coreData.Trade;

        // Get the price and net change
        double Price = NxCore.PriceToDouble(Trade->Price, Trade->PriceType);
        double NetChange = NxCore.PriceToDouble(Trade->NetChange, Trade->PriceType);

        if (Trade->ExtTradeConditions[0] != 0)
          // Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
          Console.WriteLine("Trade for Symbol: {0:S}, Time: {1:d2}:{2:d2}:{3:d2}.{4:d3}  Price: {5:f}  NetChg: {6:f}  Size: {7:d}  Exchg: {8:d}  TC: {9:d} ( {10:d}, {11:d}, {12:d}, {13:d} )",
                        Symbol,
                        pNxCoreMsg->coreHeader.nxExgTimestamp.Hour, pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
                        pNxCoreMsg->coreHeader.nxExgTimestamp.Second, pNxCoreMsg->coreHeader.nxExgTimestamp.Millisecond,
                        Price, NetChange, Trade->Size,
                        pNxCoreMsg->coreHeader.ReportingExg,
                        Trade->TradeCondition,
                        Trade->ExtTradeConditions[0], Trade->ExtTradeConditions[1], Trade->ExtTradeConditions[2], Trade->ExtTradeConditions[3]);
        else
           // Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
           Console.WriteLine("Trade for Symbol: {0:S}, Time: {1:d2}:{2:d2}:{3:d2}.{4:d3}  Price: {5:f}  NetChg: {6:f}  Size: {7:d}  Exchg: {8:d}  TC: {9:d}",
                        Symbol,
                        pNxCoreMsg->coreHeader.nxExgTimestamp.Hour, pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
                        pNxCoreMsg->coreHeader.nxExgTimestamp.Second, pNxCoreMsg->coreHeader.nxExgTimestamp.Millisecond,
                        Price, NetChange, Trade->Size,
                        pNxCoreMsg->coreHeader.ReportingExg,
                        Trade->TradeCondition);

         if (pNxCoreMsg->pnxCore3Ext != null)
         {
           JTime SRCTime,PTP1Time,PTP2Time;
           JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDaySRC, &SRCTime);
           JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDayPTP1,&PTP1Time);
           JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDayPTP2,&PTP2Time);

           Console.WriteLine("[SRC TIME: {0:d2}:{1:d2}:{2:d2}.{3:d3}.{4:d3}]  [PTP1 TIME: {5:d2}:{6:d2}:{7:d2}.{8:d3}.{9:d3}]  [PTP2 TIME: {10:d2}:{11:d2}:{12:d2}.{13:d3}.{14:d3}]", 
               SRCTime.Hour, SRCTime.Minute, SRCTime.Second, SRCTime.Millisecond,SRCTime.Microsecond,
               PTP1Time.Hour, PTP1Time.Minute, PTP1Time.Second, PTP1Time.Millisecond, PTP1Time.Microsecond,
               PTP2Time.Hour, PTP2Time.Minute, PTP2Time.Second, PTP2Time.Millisecond, PTP2Time.Microsecond);
         }
      }
	}

	
	// OnNxCoreQuote: Function to handle NxCore ExgQuote messages.	
	//--------------------------------------------------------------
	static unsafe void OnNxCoreExgQuote(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	}

	
	// OnNxCoreMMQuote: Function to handle NxCore MMQuote messages.	
	//--------------------------------------------------------------
	static unsafe void OnNxCoreMMQuote(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	}
		
  }
}
