//////////////////////////////////////////////////////
//// SampleApp9.cpp                               ////
//// Sample App for NxCore in C++                 ////
//// Author: Jeffrey Donovan, Jeremy Truelove     ////
//// Date: 2017-02-09                             ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// Sending User Defined Data through the        ////
//// ProcessTape function, retrieving it in the   ////
//// NxCore Callback.                             ////
//////////////////////////////////////////////////////
//// To Read as Written:                          ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs       ////
//////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

NxDate g_nxdates[3];

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		cout << "SampleApp09 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp09 Start." << endl;

	if (NxCore.LoadNxCore(argv[1]))
    {        
	  int dateIndex = 2;
	  g_nxdates[dateIndex].Day = 1;
      g_nxdates[dateIndex].Month = 6;
      g_nxdates[dateIndex].Year = 2010;

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  // Send in User Defined Data	
	  if (argv[2]){
        cout << "Processing the tape: " << argv[2] << endl;
		returnValue = NxCore.ProcessTape(argv[2], NULL, 0, dateIndex, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0,	dateIndex, (NxCoreCallback)OnNxCoreCallback);
                    
	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp09 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{   
  // Grab the data in UserData 
  int dateIndex = pNxCoreSys->UserData;

  // Print out custom data
  cout.fill('0');
  cout << "UserData Date: " <<
	setw(2) << (int) g_nxdates[dateIndex].Month << "/" <<
	setw(2) << (int) g_nxdates[dateIndex].Day << "/" <<
	setw(4) << (int) g_nxdates[dateIndex].Year << " " << endl;
	  
  // Stop Processing Tape
  return NxCALLBACKRETURN_STOP;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {	  
	  case NxCORESTATUS_COMPLETE:
		   cout << "NxCore Complete Message." << endl;
		   break;

      case NxCORESTATUS_INITIALIZING:
    	   cout << "NxCore Initialize Message." << endl;
    	   cout << "libnx.so version is v"
				<< NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "."
				<< NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "."
				<< NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
    	   break;

	  case NxCORESTATUS_SYNCHRONIZING:
		   cout << "NxCore Synchronizing Message." << endl;
		   break;

	  case NxCORESTATUS_WAITFORCOREACCESS:
		   cout << "NxCore Wait For Access." << endl;
		   break;

	  case NxCORESTATUS_RESTARTING_TAPE:
		   cout << "NxCore Restart Tape Message." << endl;
		   break;

      case NxCORESTATUS_ERROR:
		   cout << "NxCore Error." << endl;
		   break;
 	  
      case NxCORESTATUS_RUNNING:		   
		   break;

    }
}
