import net.nanex.NxCoreClass;
class SampleApp02 extends NxCoreClass{
	public static void main(String args[]) {
		SampleApp02 NxCore= new SampleApp02();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp02 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp02 Start.");
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, 0, 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, 0, 0);
			}
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp02 Stop.");
	}
	
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
	
	// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
	//-------------------------------------------------------------------
	void OnNxCoreSymbolSpin( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// If this is an option, and there is a valid option header, use it
		if ((NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='o')&&(NxCoreMsg.coreHeader.pnxOptionHdr != null))
		{
			// Get the strike price from header info
			double StrikePrice = NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice / 1000.0;
			
			String OptionSymbol;
			String UnderlyingStr = "";
			String SeriesStr="";
			
			// Get the DateStrike string, Series string and Underlying symbol string
			if (NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying != null)
				UnderlyingStr=NxCoreMsg.coreHeader.pnxOptionHdr.pnxsUnderlying.String;
			if (NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSeriesChain != null)
				SeriesStr=NxCoreMsg.coreHeader.pnxOptionHdr.pnxsSeriesChain.String;
			
			// If pnxsDateAndStrike.String.charAt(1) == ' ', then this symbol is in new OSI format. 
			if (NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(1)==' ')			
	    	{
				// Construct OSI Symbol, take out 'o' from symbol root (String+1)
				OptionSymbol = String.format("%-6s%02d%02d%02d%c%08d",
						NxCoreMsg.coreHeader.pnxStringSymbol.String.substring(1),
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Year-2000,
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Month,
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Day,
						(NxCoreMsg.coreHeader.pnxOptionHdr.PutCall == 0) ? 'C' : 'P',
						NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice);
				System.out.println("Symbol Spin for: " +
				" NxSym: " 			+ NxCoreMsg.coreHeader.pnxStringSymbol.String +
				" OSI Sym: " 		+ OptionSymbol +
				" Strike: " 		+ String.format("%.2f", StrikePrice) +
				" Underlying: " 	+ UnderlyingStr +
				" Series Chain: " 	+ SeriesStr);
	    	}
			else
			{
				// Otherwise the symbol is in old OPRA format.
				// Take out 'o' from symbol series (String+1)
				OptionSymbol = String.format("%s%c%c",
						NxCoreMsg.coreHeader.pnxStringSymbol.String.substring(1),
						NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(0),
				        NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(1));
				
				// Print the option symbol, underlying symbol and the series chain
				System.out.println("Symbol Spin for: " +
				" NxSym: " 			+ NxCoreMsg.coreHeader.pnxStringSymbol.String +
				" OPRA Sym: " 		+ OptionSymbol +
				" Strike: " 		+ String.format("%.2f", StrikePrice) +
				" Underlying: " 	+ UnderlyingStr +
				" Series Chain: " 	+ SeriesStr);
			}
		}
		else 
		{
			// Print the symbol and listed exchange for this spin message
			System.out.println( "Symbol Spin for: " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
				" Listed Exg: " + NxCoreMsg.coreHeader.ListedExg);
		}
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			 OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		// NxCore Symbol Spin Message
		case defines.NxMSG_SYMBOLSPIN: 
			 OnNxCoreSymbolSpin(NxCoreSys, NxCoreMsg);
			 break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}