import net.nanex.NxCoreClass;
class SampleApp03 extends NxCoreClass{
	public static void main(String args[]) {
		SampleApp03 NxCore= new SampleApp03();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp03 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp03 Start.");
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument. Use control flags
			// to eliminate Level1 quotes, Level2 quotes and all OPRA.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, (SampleApp03.defines.NxCF_EXCLUDE_QUOTES | SampleApp03.defines.NxCF_EXCLUDE_QUOTES2 | SampleApp03.defines.NxCF_EXCLUDE_OPRA), 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, (SampleApp03.defines.NxCF_EXCLUDE_QUOTES | SampleApp03.defines.NxCF_EXCLUDE_QUOTES2 | SampleApp03.defines.NxCF_EXCLUDE_OPRA), 0);
			}
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp03 Stop.");
	}
	
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
	
	// OnNxCoreCategory: Function to handle NxCore Category messages.
	// In this example we only capture category 4 and category 13 messages.
	//--------------------------------------------------------------
	void OnNxCoreCategory(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
	  // Category field
	  NxCategoryField field;
	  	  	  
	  // Do something based on the category number
		switch (NxCoreMsg.coreData.Category.pnxStringCategory.Atom)
		{		 
			// Category 4, Equity Symbol Information
			case 4:

			  // Grab a pointer to the field
			  field = NxCoreMsg.coreData.Category.pnxFields[0];

			  // Is field set?
			  if (field.Set == 1)
			  {			

				// Print the Company Name			
				  System.out.println( "Category 4 for Symbol: " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
						 " Company Name= " + field.data.StringZ);;
			  }		  
			  break;

			// Category 13, 52 Week High/Low
			case 13:
				//ensure all required fields are set.
				if(NxCoreMsg.coreData.Category.pnxFields[0].Set==0||NxCoreMsg.coreData.Category.pnxFields[1].Set==0||NxCoreMsg.coreData.Category.pnxFields[2].Set==0||NxCoreMsg.coreData.Category.pnxFields[3].Set==0)
					return;			  
				// Get 52 week high date
				NxDate HighDate=NxCoreMsg.coreData.Category.pnxFields[0].data.nxDate;
			  
				// Get 52 week low date
				NxDate LowDate=NxCoreMsg.coreData.Category.pnxFields[1].data.nxDate;
			  
				// Get the 52 week high
				double High52 = PriceToDouble(NxCoreMsg.coreData.Category.pnxFields[2].data.nxPrice.Price, NxCoreMsg.coreData.Category.pnxFields[2].data.nxPrice.PriceType);

				// Get the 52 week low
				double Low52 = PriceToDouble(NxCoreMsg.coreData.Category.pnxFields[3].data.nxPrice.Price, NxCoreMsg.coreData.Category.pnxFields[3].data.nxPrice.PriceType);

				System.out.printf("Category 13 for Symbol: %s, %02d/%02d/%04d High52= %.2f, %02d/%02d/%04d Low52= %.2f",
					NxCoreMsg.coreHeader.pnxStringSymbol.String,
					HighDate.Month, HighDate.Day, HighDate.Year, High52,
					LowDate.Month, LowDate.Day, LowDate.Year, Low52);
				break;
		  }	  
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch (NxCoreMsg.MessageType)
		{
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			 OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		// NxCore Category Message
		case defines.NxMSG_CATEGORY:
			OnNxCoreCategory(NxCoreSys,NxCoreMsg);
			break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}