import net.nanex.NxCoreClass;
class SampleApp05{
	public static void main(String args[]) throws InterruptedException {
		if(args.length < 1) {
			System.out.println("SampleApp05 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp05 Start.");
		
		//spawn a thread that will load NxCore, handle our TRADE and QUOTE messages
		//and then exit
		Thread thread1 = new Thread(new NxCoreThread(args[0],args.length > 1?args[1]:""));
		Thread thread2 = new Thread(new NxCoreThread(args[0],args.length > 1?args[1]:""));
		Thread thread3 = new Thread(new NxCoreThread(args[0],args.length > 1?args[1]:""));
		
		thread1.start();
		thread2.start();
		thread3.start();
		thread1.join();
		thread2.join();
		thread3.join();
		System.out.println("NxCore C++ SampleApp05 Stop.");
	}
}
class NxCoreThread extends NxCoreClass implements Runnable {
	private String dllFilename;
	private String filename;
	public NxCoreThread(String libPath, String tapePath){
		this.dllFilename=libPath;
		this.filename=tapePath;
	}
	public void run()
    {
		int returnValue;
		// If we can load the NxCore API
		if(LoadNxCore(dllFilename)!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			returnValue = ProcessTape(filename, 0, 0,0);
			// output return value from processTape(), helps in troubleshooting potential issues
			ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		
	}
	
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
	
	// OnNxCoreTrade: Function to handle NxCore Trade messages.	
	//--------------------------------------------------------------
	void OnNxCoreTrade( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		NxCoreTrade Trade = NxCoreMsg.coreData.Trade;

		// Get the price and net change
		double Price = PriceToDouble(Trade.Price,Trade.PriceType);
		double NetChange = PriceToDouble(Trade.NetChange, Trade.PriceType);

		// Write out Symbol, Time, Price, NetChg, Size, Reporting Exchange
		
		System.out.println( "Trade for Symbol: " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
			" Time: "+
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Hour) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Minute) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Second) +
			" Price: " 	+ String.format("%.2f", Price ) +
			" NetChg: " + String.format("%.2f", NetChange ) +
			" Size: " 	+ Trade.Size +
			" Exchg: " 	+ NxCoreMsg.coreHeader.ReportingExg );
	}
	
	// OnNxCoreQuote: Function to handle NxCore ExgQuote messages.	
	//--------------------------------------------------------------
	void OnNxCoreExgQuote( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		NxCoreExgQuote Quote = NxCoreMsg.coreData.ExgQuote;

		// Get bid and ask price
		double Bid = PriceToDouble(Quote.coreQuote.BidPrice, Quote.coreQuote.PriceType);
		double Ask = PriceToDouble(Quote.coreQuote.AskPrice, Quote.coreQuote.PriceType);

		// Write out Symbol, Time, Bid, Ask, Bid Size, Ask Size, Reporting Exchange
		System.out.println( "ExgQuote for Symbol: " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
			" Time: "+
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Hour) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Minute) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Second) +
			" Bid: " 		+ String.format("%.2f", Bid ) +
			" Ask: " 		+ String.format("%.2f", Ask ) +
			" BidSize: " 	+ Quote.coreQuote.BidSize +
			" AskSise: " 	+ Quote.coreQuote.AskSize +
			" Exchg: " 		+ NxCoreMsg.coreHeader.ReportingExg);
	}
	
	// OnNxCoreMMQuote: Function to handle NxCore MMQuote messages.	
	//--------------------------------------------------------------
	void OnNxCoreMMQuote( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		NxCoreMMQuote Quote = NxCoreMsg.coreData.MMQuote;
		  	  	  
		// Get bid and ask price
		double Bid = PriceToDouble(Quote.coreQuote.BidPrice, Quote.coreQuote.PriceType);
		double Ask = PriceToDouble(Quote.coreQuote.AskPrice, Quote.coreQuote.PriceType);

		// Write out Symbol, MarketMaker, Time, Bid, Ask, Bid Size, Ask Size, Reporting Exchange
		System.out.println( "MMQuote for Symbol: " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
		" MarketMaker: " + Quote.pnxStringMarketMaker.String +
		" Time: "+
		String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Hour) + ":" +
		String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Minute) + ":" +
		String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Second) +
		" Bid: " 		+ String.format("%.2f", Bid ) +
		" Ask: " 		+ String.format("%.2f", Ask ) +
		" BidSize: " 	+ Quote.coreQuote.BidSize +
		" AskSise: " 	+ Quote.coreQuote.AskSize +
		" Exchg: " 		+ NxCoreMsg.coreHeader.ReportingExg);
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			 OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		
		// NxCore Trade Message
		case defines.NxMSG_TRADE: 
			OnNxCoreTrade(NxCoreSys, NxCoreMsg);
			 break;
		
		// NxCore Level1 Quote Message
		case defines.NxMSG_EXGQUOTE: 
			OnNxCoreExgQuote(NxCoreSys, NxCoreMsg);
			 break;
		
		// NxCore Level2 Quote Message
		case defines.NxMSG_MMQUOTE: 
			OnNxCoreMMQuote(NxCoreSys, NxCoreMsg);
			 break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}