import net.nanex.NxCoreClass;
class SampleApp06 extends NxCoreClass{
	public static void main(String args[]) {
		SampleApp06 NxCore= new SampleApp06();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp06 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp06 Start.");
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, 0, 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, 0, 0);
			}
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp06 Stop.");
	}
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
		
	// OnNxCoreMMQuote: Function to handle NxCore MMQuote messages.	
	//--------------------------------------------------------------
	void OnGetMMStates( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		NxCoreStateMMQuotes StatMMQuotes=StateGetMMQuotes(NxCoreMsg.coreHeader.ReportingExg,NxCoreMsg.coreHeader.ListedExg,NxCoreMsg.coreHeader.pnxStringSymbol.String);
		if(StatMMQuotes!=null) 
		{
			System.out.println( "StateMMQuotes for " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
					" Count=" + StatMMQuotes.StateQuoteCount);

			// Cycle through all quotes in the state
			for (int loop = 0; loop < StatMMQuotes.StateQuoteCount; loop++)
			{
				// Get bid and ask price
				double Bid = PriceToDouble(StatMMQuotes.StateMMQuotes[loop].BidPrice, StatMMQuotes.PriceType);
				double Ask = PriceToDouble(StatMMQuotes.StateMMQuotes[loop].AskPrice, StatMMQuotes.PriceType);
				int BidSize = StatMMQuotes.StateMMQuotes[loop].BidSize;
				int AskSize = StatMMQuotes.StateMMQuotes[loop].AskSize;

				System.out.println( "Market Maker: " 	+ StatMMQuotes.StateMMQuotes[loop].pnxStringMarketMaker.String +
						  " Bid: " 			+ String.format("%.2f", Bid ) +
						  " Ask: " 			+ String.format("%.2f", Ask ) +
						  " BidSize: " 		+ BidSize +
						  " AskSize: " 		+ AskSize);
			}
		}
	}
	
	// OnNxCoreQuote: Function to handle NxCore ExgQuote messages.	
	//--------------------------------------------------------------
	void OnGetExgQuoteStates( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		NxCoreStateExgQuotes StatExgQuotes;

		// Retrieve Exchange Quote
		if ((NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='o') && NxCoreMsg.coreHeader.pnxOptionHdr!=null)// If an option....
			StatExgQuotes=StateGetExgQuotes(NxCoreMsg.coreHeader.ListedExg,NxCoreMsg.coreHeader.pnxStringSymbol.String,NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate,NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice,NxCoreMsg.coreHeader.pnxOptionHdr.PutCall);
		else// otherwise a non-option
			StatExgQuotes=StateGetExgQuotes(NxCoreMsg.coreHeader.ListedExg,NxCoreMsg.coreHeader.pnxStringSymbol.String);
		//make sure a valid return from function
		if(StatExgQuotes!=null) {
			System.out.println( "StateExgQuotes for " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
    		" BestBidExg: " + StatExgQuotes.BestBidExg +
			" BestAskExg: " + StatExgQuotes.BestAskExg);
			// Cycle through all quotes in the state
			for(int loop = 0; loop < StatExgQuotes.StateQuoteCount; loop++)
			{
				// Get bid and ask price
				double Bid = PriceToDouble(StatExgQuotes.StateExgQuotes[loop].BidPrice, StatExgQuotes.PriceType);
				double Ask = PriceToDouble(StatExgQuotes.StateExgQuotes[loop].AskPrice, StatExgQuotes.PriceType);
				int BidSize = StatExgQuotes.StateExgQuotes[loop].BidSize;
				int AskSize = StatExgQuotes.StateExgQuotes[loop].AskSize;
				int ReportingExg = StatExgQuotes.StateExgQuotes[loop].ReportingExg;
				int Condition = StatExgQuotes.StateExgQuotes[loop].QuoteCondition;

				System.out.println( "Exg: " + ReportingExg +
				  " Bid: " 			+ String.format("%.2f", Bid ) +
				  " Ask: " 			+ String.format("%.2f", Ask ) +
				  " BidSize: " 		+ BidSize +
				  " AskSize: " 		+ AskSize +
	    		  " Condition: "  	+ Condition);
			}
		}
	}


	
	// OnGetLastTradeStates: Function to retrieve the current composite trade
	// info for the issue, populated to the current time.
	//----------------------------------------------------------
	void OnGetLastTradeStates( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		NxCoreStateTrade StatLastTrade;
		
		// Retrieve Last Trade State....make sure a valid return from function
		if ((NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='o') && NxCoreMsg.coreHeader.pnxOptionHdr!=null)// If an option....
			StatLastTrade=StateGetLastTrade(NxCoreMsg.coreHeader.ListedExg,NxCoreMsg.coreHeader.pnxStringSymbol.String,NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate,NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice,NxCoreMsg.coreHeader.pnxOptionHdr.PutCall);
		else// otherwise a non-option
			StatLastTrade=StateGetLastTrade(NxCoreMsg.coreHeader.ListedExg,NxCoreMsg.coreHeader.pnxStringSymbol.String);
		
		if (StatLastTrade!=null)
		{
			System.out.println( "StateLastTrade for " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
		    	" Open: " 		+ String.format("%.2f", PriceToDouble(StatLastTrade.Open, StatLastTrade.PriceType)) +
				" High: " 		+ String.format("%.2f", PriceToDouble(StatLastTrade.High, StatLastTrade.PriceType)) +
				" Low: " 		+ String.format("%.2f", PriceToDouble(StatLastTrade.Low, StatLastTrade.PriceType)) +
				" Last: " 		+ String.format("%.2f", PriceToDouble(StatLastTrade.Last, StatLastTrade.PriceType)) +
				" Price: " 		+ String.format("%.2f", PriceToDouble(StatLastTrade.Price, StatLastTrade.PriceType)) +
				" NetChg: " 	+ String.format("%.2f", PriceToDouble(StatLastTrade.NetChange, StatLastTrade.PriceType)) +
				" TotalVol: " 	+ StatLastTrade.TotalVolume);
		}
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			 OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		// NxCore Trade Message
		case defines.NxMSG_TRADE: 
			OnGetLastTradeStates(NxCoreSys, NxCoreMsg);
			 break;
		// NxCore Level1 Quote Message
		case defines.NxMSG_EXGQUOTE: 
			OnGetExgQuoteStates(NxCoreSys, NxCoreMsg);
			 break;
		// NxCore Level2 Quote Message
		case defines.NxMSG_MMQUOTE: 
			OnGetMMStates(NxCoreSys, NxCoreMsg);
			 break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}