import net.nanex.NxCoreClass;
import java.io.IOException;
class SampleApp07 extends NxCoreClass{
	public static void main(String args[]) {
		SampleApp07 NxCore= new SampleApp07();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp07 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp07 Start.");
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, 0, 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, 0, 0);
			}
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp07 Stop.");
	}
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		// Capture a SYMBOL SPIN Message
		case defines.NxCORESTATUS_SYMBOLSPIN:
			switch (NxCoreSys.StatusData) {

			// If initial symbol spin is complete, test the functions in this
			// sample application
			case defines.NxCSSYMBOLSPIN_COMPLETE:
				System.console().readLine();
				// User defined symbol spin
				System.out.println("NxCore Initial Spin Complete Message.");
				System.out.println("Press <Enter> to run a user define spin.");
				try {
					System.in.read();
				} catch (IOException e) {}

				SpinSymbols(defines.NxSPIN_ALL_EXGS, 0, 1, 0);
				
				System.out.println("UserDefine Spin Complete!");
				
				//Get Available NxCore Data file listings
				System.out.println("Press <Enter> to get available NxCore data files.");
				System.console().readLine();

				ListTapes(0);

				//Get Available NxCore API listings
				System.out.println("Press <Enter> to get available NxCore API files.");
				System.console().readLine();

				ListAPIDLLs(0);
				
				// Get defined string from NxCore tables
				System.out.println("Press <Enter> to get defined strings for exchanges.");
				System.console().readLine();

				PrintDefinedString();

				// Get current tape filename
				System.out.println("Press <Enter> to get the Tape Name.");
				System.console().readLine();

				PrintTapeName();

				// Get date from N days
				System.out.println("Press <Enter> to get a date from N days.");
				System.console().readLine();

				PrintDateFromDays();

				// Get N Days from date
				System.out.println("Press <Enter> to get NDays from a date.");
				System.console().readLine();

				PrintDaysFromDate();

				// Finish!
				System.out.println("Press <Enter> to continue with tape.");
				System.console().readLine();
			}
			break;
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
	
	// GetDefinedString: Get string description from NxCore table definitions.
	//------------------------------------------------------------------------
	void PrintDefinedString()
	{
		// Loop for 60 exchanges...
	 	for (int loop = 1; loop < 68; loop++)
		{

	 		// Get the defined name for this exchange
	 		String Exg = GetDefinedString((int)defines.NxST_EXCHANGE, loop);		

	 		// Print exchange num and name
	 		System.out.println("Exg #" + loop +
				" = " 	+ Exg);
	  }
	}


	// GetTapeName: Get filename for current tape
	//-------------------------------------------
	void PrintTapeName()
	{
		  
		// Get current tape name, convert to string type
		String TapeName = GetTapeName();
	  
		// Print tape name
		System.out.println( "TapeName " + TapeName );
	}


	// DateFromDays: Get date from N Days
	//-----------------------------------
	void PrintDateFromDays()
	{
		NxDate Date = new NxDate();
		  
		// Set NDays to get date for
		Date.NDays=32000;

		// Call DateFromNDays function
		DateFromNDays(Date);

		// print NDays and date
		System.out.println( "Date for " + Date.NDays + " Days: " +
			String.format("%02d",Date.Month) + "/" +
			String.format("%02d",Date.Day) + "/" +
			String.format("%04d",Date.Year));
	}


	// DaysFromDate: Get N days from date
	//-----------------------------------
	void PrintDaysFromDate()
	{
		NxDate Date = new NxDate();
		
		// Set date to get NDays from
		Date.Month=1;
		Date.Day  =1;
		Date.Year = 2010;
		
		// Call DateFromYMD function
		DateFromYMD(Date);
	  
		// Print date and NDays
		System.out.println( "Date " +
			String.format("%02d",Date.Month) + "/" +
			String.format("%02d",Date.Day) + "/" +
			String.format("%04d",Date.Year) +
			" Days: " + Date.NDays);
	}


	// OnUserListTapes: Callback with abailable tape filenames on sytstem
	//-------------------------------------------------------------------
	@Override
	public int OnUserListTapes(NxCoreTapeFile NxTF)
	{
		// Print the tape filename
		System.out.println("Filename: " + NxTF.PathnameStrZ);
		  
	  // Continue to next
	  return defines.NxCALLBACKRETURN_CONTINUE;
	}


	// OnUserListAPIDLLs: Callback with abailable NxCore DLL's on sytstem
	//-------------------------------------------------------------------
	@Override
	public int OnUserListAPI(NxCoreAPIDLLFile NxDF)
	{
		// Print the DLL filename
		System.out.println( "Filename: " + NxDF.PathnameStrZ);

		// Continue to next
		return defines.NxCALLBACKRETURN_CONTINUE;
	}


	// OnUserSymbolSpin:  User defined symbol spin callback
	//------------------------------------------------------
	@Override
	public int OnUserSymbolSpin( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{

		// Print the spin message sym, exg, time, date and session ID	  
		System.out.println( "UserSymbol Spin for: " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
			" Listed Exg: " + NxCoreMsg.coreHeader.ListedExg +
			" Time: " +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Hour) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Minute) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Second) + " " +

			String.format("%02d", NxCoreMsg.coreHeader.nxSessionDate.Month) + "/" +
			String.format("%02d",  NxCoreMsg.coreHeader.nxSessionDate.Day) + "/" +
			String.format("%04d",  NxCoreMsg.coreHeader.nxSessionDate.Year) +
			NxCoreMsg.coreHeader.SessionID);

	  // Continue to next
	  return defines.NxCALLBACKRETURN_CONTINUE;
	}

		
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}