import net.nanex.NxCoreClass;
class SampleApp08 extends NxCoreClass{
	int MainCounter = 0;
	int MainOptionCounter = 0;
	boolean IsOSI=false;
	public static void main(String args[]) {
		SampleApp08 NxCore= new SampleApp08();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp08 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp08 Start.");
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, 0, 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, 0, 0);
			}
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp08 Stop.");
	}
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		
		case defines.NxCORESTATUS_SYMBOLSPIN:
			switch (NxCoreSys.StatusData)
			{

				// Capture a SYMBOL SPIN Message
				// If initial symbol spin is complete, test the functions in this
				// sample application 
				case defines.NxCSSYMBOLSPIN_COMPLETE:

					//NxCore.SpinSymbols(NxSPIN_ALL_EXGS, 0, 1, (NxCoreCallback)OnUserSymbolSpin_NonOptions, 0);

					// User defined symbol spin
					System.out.println( "NxCore User Spin Complete Message." );
					System.out.println("Press <Enter> to run a user define spin for options." );
					System.console().readLine();

					// If the tape is an OSI tape, point the spin to the OSI handler,
					// otherwise point the spin to the Traditional OPRA handler
					if (IsOSI)
						SpinSymbols(defines.NxSPIN_ALL_EXGS, defines.NxSPIN_OPTIONS, 1, 0);
					else
						SpinSymbols(defines.NxSPIN_ALL_EXGS, defines.NxSPIN_OPTIONS, 1, 0);

				// Finish!				
					System.out.println( "Press <Enter> to continue with tape." );
					System.console().readLine();
				break;
			}
			break;
			
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
		case defines.NxCORESTATUS_TAPEOPEN:
			int Major = GetMajorVersion(NxCoreSys.DLLVersion);
			int Minor = GetMinorVersion(NxCoreSys.DLLVersion);
			int Sub = GetBuildVersion(NxCoreSys.DLLVersion);

				System.out.println( "NxCore Tape Open Message." );
				System.out.println( " Tape Version Numeric: v" + Major + "." +
						Minor + "." +
						Sub +
						" System Date: " +
						String.format("%02d", NxCoreSys.nxDate.Month) + "/" +
						String.format("%02d",  NxCoreSys.nxDate.Day) + "/" +
						String.format("%04d",  NxCoreSys.nxDate.Year));

				// If tape version is greater than 1.20.x then this is an OSI tape
				if (Major>1||(Major==1 && Minor>20)) IsOSI=true;

				System.out.println( "Press <Enter> to Continue..." );
				System.console().readLine();
			
		}
	}
	
	// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
	//-------------------------------------------------------------------
	void OnNxCoreSymbolSpin( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// If this is an option, and there is a valid option header, use it
		if ((NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='o')&&(NxCoreMsg.coreHeader.pnxOptionHdr != null))
		{
			String OptionSymbol;

			// SET UserData1 for the option to MainOptionCounter (increment the counter as we do).
			int UData1=NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData1 = ++MainOptionCounter;

			// SET UserData2 to some odd value based on Nx Clock
			int UData2=NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData2 = NxCoreSys.nxTime.Second * NxCoreSys.nxTime.Millisecond;
			
			// If tape is determined to be an OSI tape...
			if (IsOSI)
			{
				// Construct OSI Symbol, take out 'o' from symbol root (String+1)
				OptionSymbol = String.format("%-6s%02d%02d%02d%c%08d",
						NxCoreMsg.coreHeader.pnxStringSymbol.String.substring(1),
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Year-2000,
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Month,
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Day,
						(NxCoreMsg.coreHeader.pnxOptionHdr.PutCall == 0) ? 'C' : 'P',
						NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice);
				System.out.println("Initial Symbol Spin for: NxSym: " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
				" OSI Sym: " 		+ OptionSymbol +
				" UserData1: " 	+ UData1 +
				" UserData2: " 	+ UData2);
			}
			else
			{
				OptionSymbol = String.format("%s%c%c",
						NxCoreMsg.coreHeader.pnxStringSymbol.String.substring(1),
						NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(0),
						NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(1));
				
				// Print the option symbol, underlying symbol and the series chain
				System.out.println("Initial Symbol Spin for: NxSym: " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
				" OPRA Sym: " 	+ OptionSymbol +
				" UserData1: " 	+ UData1 +
				" UserData2: " 	+ UData2);
			}
		}
	}
	
	// OnUserSymbolSpin:  User defined symbol spin callback for tape with option symbols
	//------------------------------------------------------
	@Override
	public int OnUserSymbolSpin( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		if ((NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='o')&&(NxCoreMsg.coreHeader.pnxOptionHdr != null))
		{
			String OptionSymbol;
			
			// Retrieve the UseerData1 and UserData2 values set in the initial spin
			int UData1 = NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData1;
			int UData2 = NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.UserData2;
			if(IsOSI) {
				OptionSymbol = String.format("%s%c%c",
						NxCoreMsg.coreHeader.pnxStringSymbol.String.substring(1),
						NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(0),
						NxCoreMsg.coreHeader.pnxOptionHdr.pnxsDateAndStrike.String.charAt(1));
			}
			else {
				// Construct OSI Symbol, take out 'o' from symbol root (String+1)
				OptionSymbol = String.format("%-6s%02d%02d%02d%c%08d",
						NxCoreMsg.coreHeader.pnxStringSymbol.String.substring(1),
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Year-2000,
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Month,
						NxCoreMsg.coreHeader.pnxOptionHdr.nxExpirationDate.Day,
						(NxCoreMsg.coreHeader.pnxOptionHdr.PutCall == 0) ? 'C' : 'P',
						NxCoreMsg.coreHeader.pnxOptionHdr.strikePrice);
			}
			// Print the symbol and listed exchange for this spin message
			System.out.println( "User Symbol Spin for: " + NxCoreMsg.coreHeader.pnxStringSymbol.String +
					" OPRA Sym: " + OptionSymbol +
					" UserData1=" + UData1 +
					" UserData2=" + UData2 );
		}
	  // Continue to next
	  return defines.NxCALLBACKRETURN_CONTINUE;
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			break;
		// NxCore Symbol Spin Message 
		case defines.NxMSG_SYMBOLSPIN: 
			OnNxCoreSymbolSpin(NxCoreSys, NxCoreMsg);
			break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}