import net.nanex.NxCoreClass;
class SampleApp10 extends NxCoreClass{
	int SavedState=0;
	String StateFileFilename;
	public static void main(String args[]) {
		SampleApp10 NxCore= new SampleApp10();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp10 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp10 Start.");
		
		NxCore.StateFileFilename="STATESAVE.TMP";
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			
			System.out.println( "The application will process trades until 7:00 AM and then save the state," );
			System.out.println( "exit and reload from the state tape." );
			System.out.println( "Press <ENTER> to continue." );
			System.console().readLine();
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, 0, 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, 0, 0);
			}

			// The current process has exited and the state tape has been saved!
			System.out.println("Tape state saved and process exited.");
			System.out.println("Press <ENTER> to Load the state and continue." );
			System.console().readLine();
			
			// Reset this flag so we dont stop in the next call to ProcessTape
			NxCore.SavedState=0;
			
			// Process the tape from a state file
			returnValue = NxCore.ProcessTape(NxCore.StateFileFilename, 0, 0, 0);
			
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp10 Stop.");
	}
	
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
			
			// LOOK! If it's 7:00:00 AM SAVE THE CURRENT STATE TO A STATE FILE!
			if (NxCoreSys.nxTime.Hour==7 && NxCoreSys.nxTime.Minute==0 && NxCoreSys.nxTime.Second==0)
			{
				// NOTE: This application will only work properly with the NxSAVESTATE_ONEPASS flag setting,
				// as we exit the tape exactly one pass after the file is to be saved. Should you modify this
				// flag the application will need to also be modified accordingly.
				SaveState(StateFileFilename,defines.NxSAVESTATE_ONEPASS);
			}
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		
		  // NxCore will save the state this pass. Tell user and wait for keypress
		case defines.NxCORESTATUS_SAVING_STATE:
	  		System.out.println( "NxCore Will Save the State Now!");
	  		System.out.println( "Press <ENTER> to Continue.");
	  		System.console().readLine();

	  		// Set the state saved flag
	  		SavedState=1;
	  		break;

		  // NxCore Has been loaded from a state tape. Tell user and wait for keypress
		case defines.NxCORESTATUS_LOADED_STATE:
	  		System.out.println( "NxCore Has Been Loaded From A State!");
	  		System.out.println("Press <ENTER> to continue processing trades.");
	  		System.console().readLine();
	  		break;
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
		
	// OnNxCoreTrade: Function to handle NxCore Trade messages.	
	//--------------------------------------------------------------
	void OnNxCoreTrade( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		if (NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='e')
		{
			NxCoreTrade Trade = NxCoreMsg.coreData.Trade;

			// Get the price and net change
			double Price = PriceToDouble(Trade.Price,Trade.PriceType);
			double NetChange = PriceToDouble(Trade.NetChange, Trade.PriceType);

			// Write out Symbol, Time, Price, NetChg, Size, Reporting Exchange
		
			System.out.println( "Trade for Symbol: " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
					" Time: "+
					String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Hour) + ":" +
					String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Minute) + ":" +
					String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Second) +
					" Price: " 	+ String.format("%.2f", Price ) +
					" NetChg: " + String.format("%.2f", NetChange ) +
					" Size: " 	+ Trade.Size +
					" Exchg: " 	+ NxCoreMsg.coreHeader.ReportingExg );
		}
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		
		// If this flag is set, NxCore has saved the state. Note that from the time
		// the NxCORESTATUS_SAVING_STATE message was received, the function issued
		// one moreNxCALLBACKRETURN_CONTINUE, and the next time in we will exit, thus
		// allowing NxCore to properly save the state and to exit automatically.
		if (SavedState!=0)
			return defines.NxCALLBACKRETURN_STOP;
		
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			 OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		// NxCore Trade Message
		case defines.NxMSG_TRADE: 
			//OnNxCoreTrade(NxCoreSys, NxCoreMsg);
			 break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}