import net.nanex.NxCoreClass;
class SampleApp11 extends NxCoreClass{
	// JTime Time Class
	class JTime
	{
	  int Hour;
	  int Minute;
	  int Second;
	  int Millisecond;
	  int Microsecond;
	  int MsOfDay;
	  long MicrosOfDay;
	};
	
	public static void main(String args[]) {
		SampleApp11 NxCore= new SampleApp11();
		int returnValue;
		if(args.length < 1) {
			System.out.println("SampleApp11 dll-file [tape-name]");
			return;
		}
		System.out.println("NxCore Java SampleApp11 Start.");
		
		// If we can load the NxCore API
		if(NxCore.LoadNxCore(args[0])!=0) {
			// If a tape filename was passed in command line argument,
			// call ProcessTape with that argument.
			if (args.length > 1){
				System.out.println("Processing the tape: " + args[1]);
				returnValue = NxCore.ProcessTape(args[1], 0, 0, 0);
			} else {
				returnValue = NxCore.ProcessTape("", 0, 0, 0);
			}
			// output return value from processTape(), helps in troubleshooting potential issues
			NxCore.ProcessReturnValue(returnValue);
			
			// Unload NxCore DLL
			NxCore.UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		System.out.println("NxCore C++ SampleApp11 Stop.");
	}
	
	void OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		// If a Minute has elapsed print the NxCore system time.
		// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
		// a real time value
		if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE) &&(NxCoreSys.nxTime.Hour < 24))
		{
			System.out.printf("%02d/%02d/%04d %02d:%02d:%02d\n",
					NxCoreSys.nxDate.Month,
					NxCoreSys.nxDate.Day,
					NxCoreSys.nxDate.Year,
					NxCoreSys.nxTime.Hour,
					NxCoreSys.nxTime.Minute,
					NxCoreSys.nxTime.Second);
		}

		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
		case defines.NxCORESTATUS_COMPLETE:
			System.out.println("NxCore Complete Message.");
			break;

		case defines.NxCORESTATUS_INITIALIZING:
			System.out.println("NxCore Initialize Message.");
			System.out.println("API version is v" + GetMajorVersion(NxCoreSys.DLLVersion) + "." + GetMinorVersion(NxCoreSys.DLLVersion) + "." + GetBuildVersion(NxCoreSys.DLLVersion) );
			break;

		case defines.NxCORESTATUS_SYNCHRONIZING:
			System.out.println("NxCore Synchronizing Message.");
			break;

		case defines.NxCORESTATUS_WAITFORCOREACCESS:
			System.out.println("NxCore Wait For Access.");
			break;

		case defines.NxCORESTATUS_RESTARTING_TAPE:
			System.out.println("NxCore Restart Tape Message.");
			break;

		case defines.NxCORESTATUS_ERROR:
			System.out.println("NxCore Error.");
			break;

		case defines.NxCORESTATUS_RUNNING:
			break;
			
		}
	}
	
	// Get Micro Diff from MsOfDay
	int JTime_MicroDiffFromMSofDay(long micros,int ms) {
		return (int)(micros-(long)ms*1000);
	}

	// Convert MicrosecondOfDay to JTime
	void JTime_MicrosOfDayToTime(long MicrosOfDay,JTime Time)
	{
	   Time.MicrosOfDay=MicrosOfDay;
	   
	   Time.MsOfDay=(int)(MicrosOfDay/1000);   
	   int Seconds=Time.MsOfDay/1000;
	     
	   Time.Hour       = Seconds/(60*60);
	   Time.Minute     = Seconds/60%60;
	   Time.Second     = Seconds%60;
	   Time.Millisecond= Time.MsOfDay % 1000;
	   Time.Microsecond= JTime_MicroDiffFromMSofDay(MicrosOfDay,Time.MsOfDay);         
	}
	
	// OnNxCoreTrade: Function to handle NxCore Trade messages.	
	//--------------------------------------------------------------
	void OnNxCoreTrade( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg)
	{
		if(NxCoreMsg.coreHeader.pnxStringSymbol.String.charAt(0)=='e') {
		NxCoreTrade Trade = NxCoreMsg.coreData.Trade;

		// Get the price and net change
		double Price = PriceToDouble(Trade.Price,Trade.PriceType);
		double NetChange = PriceToDouble(Trade.NetChange, Trade.PriceType);

		// Write out Symbol, Time, Price, NetChg, Size, Reporting Exchange
		
		System.out.printf( "Trade for Symbol: " + NxCoreMsg.coreHeader.pnxStringSymbol.String + "," +
			" Time: "+
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Hour) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Minute) + ":" +
			String.format("%02d", NxCoreMsg.coreHeader.nxExgTimestamp.Second) +
			" Price: " 	+ String.format("%.2f", Price ) +
			" NetChg: " + String.format("%.2f", NetChange ) +
			" Size: " 	+ Trade.Size +
			" Exchg: " 	+ NxCoreMsg.coreHeader.ReportingExg +
			" TC: ( " 	+ Trade.TradeCondition +")" );
		if(Trade.ExtTradeConditions[0]!=0)
		System.out.printf( " (" +
				Trade.ExtTradeConditions[0] + "," +
				Trade.ExtTradeConditions[1] + "," +
				Trade.ExtTradeConditions[2] + "," +
				Trade.ExtTradeConditions[3] + ")");
		System.out.printf("\n");
		
	    // add any microsecond timestamps (.nx3 format ONLY)
	    if (NxCoreMsg.pnxCore3Ext!=null)
	    {
	      JTime SRCTime=new JTime();
	      JTime PTP1Time=new JTime();
	      JTime PTP2Time=new JTime();
	      JTime_MicrosOfDayToTime(NxCoreMsg.pnxCore3Ext.MicrosOfDaySRC, SRCTime);
	      JTime_MicrosOfDayToTime(NxCoreMsg.pnxCore3Ext.MicrosOfDayPTP1,PTP1Time);
	      JTime_MicrosOfDayToTime(NxCoreMsg.pnxCore3Ext.MicrosOfDayPTP2,PTP2Time);

		System.out.println( "[SRC TIME: " +
			String.format("%.2f", SRCTime.Hour) + ":" +
			String.format("%.2f", SRCTime.Minute) + ":" +
			String.format("%.2f", SRCTime.Second) + "." +
			String.format("%.3f", SRCTime.Millisecond) + "." +
			String.format("%.3f", SRCTime.Microsecond) + "] " +
			"  [PTP1 TIME: " +
			String.format("%.2f", PTP1Time.Hour) + ":" +
			String.format("%.2f", PTP1Time.Minute) + ":" +
			String.format("%.2f", PTP1Time.Second) + "." +
			String.format("%.3f", PTP1Time.Millisecond) + "." +
			String.format("%.3f", PTP1Time.Microsecond) + "] " +
			"  [PTP2 TIME: " +
			String.format("%.2f", PTP2Time.Hour) + ":" +
			String.format("%.2f", PTP2Time.Minute) + ":" +
			String.format("%.2f", PTP2Time.Second) + "." +
			String.format("%.3f", PTP2Time.Millisecond) + "." +
			String.format("%.3f", PTP2Time.Microsecond) + "] ");
	    }
		}
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		switch(NxCoreMsg.MessageType) {
		// NxCore Status Message
		case defines.NxMSG_STATUS:
			 OnNxCoreStatus(NxCoreSys,NxCoreMsg);
			 break;
		// NxCore Symbol Spin Message
		case defines.NxMSG_TRADE: 
			OnNxCoreTrade(NxCoreSys, NxCoreMsg);
			 break;
		}
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}