import net.nanex.NxCoreClass;
class SampleApp12{
	static final int MAX_THREADS = 10;
	public static void main(String args[]) throws InterruptedException {
		Thread[] threads =  new Thread[MAX_THREADS];
		int numThreads=-1;
		if(args.length < 2) {
			System.out.println("SampleApp12 dll-file thread-count [tape-name]");
			return;
		}
        try {
        	numThreads = Integer.parseInt(args[1]);
        }
        catch (NumberFormatException e) {
            System.out.println("Number of threads must be between 1 and 10");
        }
        if(numThreads<0||numThreads>MAX_THREADS) {
        	System.out.println("Number of threads must be between 1 and 10");
        	return;
        }
        
		System.out.println("NxCore Java SampleApp12 Start.");
		
		for(int i = 0; i < numThreads; ++i) {
			threads[i] = new Thread(new NxCoreThreadProc(args[0],args.length > 2?args[2]:"",i));
			threads[i].start();
		}
		for(int i = 0; i < numThreads; ++i) {
			threads[i].join();
		}
		System.out.println("NxCore C++ SampleApp12 Stop.");
	}
}
class NxCoreThreadProc extends NxCoreClass implements Runnable {
	private String dllFilename;
	private String filename;
	private String stateFilename;
	private int threadNum;
	private boolean tapeComplete;
	private boolean savingState;
	public NxCoreThreadProc(String libPath, String tapePath, int num){
		this.dllFilename = libPath;
		this.filename = tapePath;
		this.threadNum = num;
	}
	String PrintTime(NxCoreSystem NxCoreSys) {
		String timeBuf = String.format("Thread %d: NxCoreTime: %02d/%02d/%02d %02d:%02d:%02d ", threadNum,
				NxCoreSys.nxDate.Month, NxCoreSys.nxDate.Day, NxCoreSys.nxDate.Year % 100,
				NxCoreSys.nxTime.Hour, NxCoreSys.nxTime.Minute, NxCoreSys.nxTime.Second);
		return timeBuf;
	}
	public void run()
    {
		int returnValue;
		stateFilename=String.format("%s.%d", filename.length()>0 ? filename : "Current",threadNum);
		tapeComplete=false;
		savingState=false;
		// If we can load the NxCore API
		if(LoadNxCore(dllFilename)!=0) {
			while(!tapeComplete) {
				returnValue = ProcessTape(filename, 0, 0,0);
				filename = stateFilename;
				System.out.println( "Thread " + threadNum + " completed: Status follows...");
				ProcessReturnValue(returnValue);
			}
			// Unload NxCore DLL
			UnloadNxCore();
		}
		else {
			System.out.println("Failed to load NxCore Library");
			return;
		}
		
	}
	
	int OnNxCoreStatus(NxCoreSystem NxCoreSys,NxCoreMessage NxCoreMsg)
	{
		
		// Print the specific NxCore status message
		switch (NxCoreSys.Status) {
			case defines.NxCORESTATUS_COMPLETE:
				tapeComplete = true;
				System.out.println(  "Thread " + threadNum + ": NxCore Complete Message.");
				break;

			case defines.NxCORESTATUS_INITIALIZING:
				System.out.println( "Thread " + threadNum + ": NxCore Initialize Message. libnx.so version is v"
					 + GetMajorVersion(NxCoreSys.DLLVersion) + "."
					 + GetMinorVersion(NxCoreSys.DLLVersion) + "."
					 + GetBuildVersion(NxCoreSys.DLLVersion));
				break;

			case defines.NxCORESTATUS_WAITFORCOREACCESS:
				System.out.println( PrintTime(NxCoreSys) + ": NxCore Wait For Access.");
				break;

			case defines.NxCORESTATUS_ERROR:
				System.out.println( PrintTime(NxCoreSys) + ": NxCore Error.");
				break;

			case defines.NxCORESTATUS_LOADED_STATE:
				System.out.println( PrintTime(NxCoreSys) + ": Loaded from state file " + stateFilename );
				break;

			case defines.NxCORESTATUS_SAVING_STATE:
				switch(NxCoreSys.StatusData) {
				case defines.NxCSSAVESTATE_CAPTURE:
					savingState = true;
					System.out.println( PrintTime(NxCoreSys) + ": Saving state file " + stateFilename );
					break;
				case defines.NxCSSAVESTATE_COMPLETE:
					savingState = false;
					System.out.println( PrintTime(NxCoreSys) + ": State file " + stateFilename + " completed");
					return defines.NxCALLBACKRETURN_STOP;
				case defines.NxCSSAVESTATE_ERR_CREATEFILE:
					savingState = false;
					System.out.println( PrintTime(NxCoreSys) + ": Save state error: CreateFile " + stateFilename );
					break;
				case defines.NxCSSAVESTATE_ERR_DISKSPACE:
					savingState = false;
					System.out.println( PrintTime(NxCoreSys) + ": Save state error: ErrDiskspace " );
					break;
				case defines.NxCSSAVESTATE_ERR_MEMORY:
					savingState = false;
					System.out.println( PrintTime(NxCoreSys) + ": Save state error: ErrMemory " );
					break;
				}
				break;

			default:
				if ((NxCoreSys.ClockUpdateInterval >= defines.NxCLOCK_MINUTE)	&& (NxCoreSys.nxTime.Hour < 24)) {
					System.out.println( PrintTime(NxCoreSys));
					if(!savingState && NxCoreSys.nxTime.Minute % 30 == 0) {
						savingState = true;
						SaveState(stateFilename, (threadNum % 2!=0) ? defines.NxSAVESTATE_ONEPASS : defines.NxSAVESTATE_GRADUALLY);
					}
				}
		}
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
	
	@Override
	public int OnNxCoreCallback( NxCoreSystem NxCoreSys, NxCoreMessage NxCoreMsg){
		// Do something based on the message type
		if (NxCoreMsg.MessageType==defines.NxMSG_STATUS)
			 return OnNxCoreStatus(NxCoreSys,NxCoreMsg);
		// Continue running the tape
		return defines.NxCALLBACKRETURN_CONTINUE;
	}
}